searchData={"items":[{"type":"task","title":"mix stability_test","doc":"Runs stability tests for WebsockexAdapter with Deribit integration.","ref":"Mix.Tasks.StabilityTest.html"},{"type":"task","title":"Usage - mix stability_test","doc":"Run 1-hour development stability test:\n    mix stability_test\n    \nRun 24-hour production stability test:\n    mix stability_test --full\n    \nRun with custom duration (in hours):\n    mix stability_test --hours 6","ref":"Mix.Tasks.StabilityTest.html#module-usage"},{"type":"module","title":"WebsockexAdapter","doc":"WebsockexAdapter is a robust WebSocket client library for Elixir with a pluggable adapter architecture.","ref":"WebsockexAdapter.html"},{"type":"module","title":"Architecture Overview - WebsockexAdapter","doc":"WebsockexAdapter employs a \"thin adapter\" architecture that separates concerns through:\n\n1. **Behavioral Interfaces**: Well-defined behaviors for various aspects of WebSocket handling\n2. **Default Implementations**: Ready-to-use default implementations of these behaviors\n3. **Platform Adapters**: Thin adapters that bridge to specific platforms/services\n4. **Connection Management**: Process-based connection handling with ownership semantics\n\nThis modular design allows for maximum flexibility while minimizing boilerplate code.","ref":"WebsockexAdapter.html#module-architecture-overview"},{"type":"module","title":"Key Components - WebsockexAdapter","doc":"* **Connection**: The core GenServer process managing the WebSocket lifecycle\n* **Client**: A convenient API for interacting with connections\n* **Behaviors**: Interfaces for connection, message, authentication, error handling, etc.\n* **Defaults**: Ready-to-use implementations of all behaviors\n* **Platform Adapters**: Thin adapters for specific WebSocket services","ref":"WebsockexAdapter.html#module-key-components"},{"type":"module","title":"Basic Usage - WebsockexAdapter","doc":"```elixir\n# Start a connection to the Echo service\n{:ok, conn} = WebsockexAdapter.Connection.start_link(\n  adapter: WebsockexAdapter.Platform.Echo.Adapter\n)\n\n# Send a message and get the response\n{:text, response} = WebsockexAdapter.Client.send_text(conn, \"Hello\")\n```","ref":"WebsockexAdapter.html#module-basic-usage"},{"type":"module","title":"Using with Custom Handlers - WebsockexAdapter","doc":"```elixir\n# Start a connection with custom handlers\n{:ok, conn} = WebsockexAdapter.Connection.start_link(\n  adapter: WebsockexAdapter.Platform.Echo.Adapter,\n  message_handler: MyApp.MessageHandler,\n  connection_handler: MyApp.ConnectionHandler\n)\n```","ref":"WebsockexAdapter.html#module-using-with-custom-handlers"},{"type":"module","title":"Creating a Client Module - WebsockexAdapter","doc":"```elixir\ndefmodule MyApp.WebSocketClient do\n  use GenServer\n\n  def start_link(opts \\\\ []) do\n    GenServer.start_link(__MODULE__, opts)\n  end\n\n  def init(opts) do\n    # Start the WebSocket connection\n    {:ok, conn} = WebsockexAdapter.Connection.start_link(\n      adapter: WebsockexAdapter.Platform.Echo.Adapter\n    )\n\n    {:ok, %{conn: conn}}\n  end\n\n  # API functions\n  def send_message(client, message) do\n    GenServer.call(client, {:send, message})\n  end\n\n  # Callbacks\n  def handle_call({:send, message}, _from, %{conn: conn} = state) do\n    result = WebsockexAdapter.Client.send_text(conn, message)\n    {:reply, result, state}\n  end\nend\n```","ref":"WebsockexAdapter.html#module-creating-a-client-module"},{"type":"module","title":"Implementing Custom Handlers - WebsockexAdapter","doc":"Each aspect of WebSocket communication can be customized by implementing one of\nthe behaviors in `WebsockexAdapter.Behaviors`:\n\n```elixir\ndefmodule MyApp.MessageHandler do\n  @behaviour WebsockexAdapter.Behaviors.MessageHandler\n\n  @impl true\n  def init(opts) do\n    {:ok, %{messages: []}}\n  end\n\n  @impl true\n  def handle_message(frame_type, data, state) do\n    Logger.debug(\"Received #{frame_type} message: #{inspect(data)}\")\n    new_state = update_in(state.messages, &[{frame_type, data} | &1])\n    {:ok, new_state}\n  end\nend\n```","ref":"WebsockexAdapter.html#module-implementing-custom-handlers"},{"type":"module","title":"Creating a Platform Adapter - WebsockexAdapter","doc":"To support a new WebSocket service, implement the `WebsockexAdapter.Platform.Adapter` behavior:\n\n```elixir\ndefmodule MyApp.CustomAdapter do\n  use WebsockexAdapter.Platform.Adapter,\n    default_host: \"api.example.com\",\n    default_port: 443,\n    default_path: \"/websocket\"\n\n  @impl true\n  def handle_platform_message(message, state) do\n    # Custom message handling\n    {:reply, {:text, \"Processed: #{inspect(message)}\"}, state}\n  end\n\n  @impl true\n  def encode_auth_request(credentials) do\n    {:text, Jason.encode!(%{\n      type: \"auth\",\n      key: credentials.api_key,\n      secret: credentials.api_secret\n    })}\n  end\n\n  @impl true\n  def encode_subscription_request(channel, params) do\n    {:text, Jason.encode!(%{\n      type: \"subscribe\",\n      channel: channel,\n      params: params\n    })}\n  end\n\n  @impl true\n  def encode_unsubscription_request(channel) do\n    {:text, Jason.encode!(%{\n      type: \"unsubscribe\",\n      channel: channel\n    })}\n  end\nend\n```","ref":"WebsockexAdapter.html#module-creating-a-platform-adapter"},{"type":"module","title":"Available Behaviors - WebsockexAdapter","doc":"* `ConnectionHandler`: Handle connection lifecycle events\n* `MessageHandler`: Process incoming WebSocket messages\n* `SubscriptionHandler`: Manage channel subscriptions\n* `AuthHandler`: Handle authentication\n* `ErrorHandler`: Process error scenarios\n* `RateLimitHandler`: Implement rate limiting\n* `LoggingHandler`: Provide logging functionality\n* `MetricsCollector`: Collect metrics about WebSocket operations\n\nEach behavior has a corresponding default implementation in the `WebsockexAdapter.Defaults` namespace.","ref":"WebsockexAdapter.html#module-available-behaviors"},{"type":"module","title":"WebsockexAdapter.Client","doc":"WebSocket client GenServer using Gun as transport layer.","ref":"WebsockexAdapter.Client.html"},{"type":"module","title":"Overview - WebsockexAdapter.Client","doc":"The Client module is implemented as a GenServer to handle asynchronous Gun messages.\nGun sends all WebSocket messages to the process that opens the connection, so the\nClient GenServer owns the Gun connection to receive these messages directly.","ref":"WebsockexAdapter.Client.html#module-overview"},{"type":"module","title":"Public API - WebsockexAdapter.Client","doc":"Despite being a GenServer internally, the public API returns struct-based responses\nfor backward compatibility:\n\n    {:ok, client} = Client.connect(\"wss://example.com\")\n    # client is a struct with gun_pid, stream_ref, and server_pid fields\n    \n    :ok = Client.send_message(client, \"hello\")\n    Client.close(client)","ref":"WebsockexAdapter.Client.html#module-public-api"},{"type":"module","title":"Connection Ownership and Reconnection - WebsockexAdapter.Client","doc":"","ref":"WebsockexAdapter.Client.html#module-connection-ownership-and-reconnection"},{"type":"module","title":"Initial Connection - WebsockexAdapter.Client","doc":"When you call `connect/2`, a new Client GenServer is started which:\n1. Opens a Gun connection from within the GenServer \n2. Receives all Gun messages (gun_ws, gun_up, gun_down, etc.)\n3. Returns a client struct containing the GenServer PID","ref":"WebsockexAdapter.Client.html#module-initial-connection"},{"type":"module","title":"Automatic Reconnection - WebsockexAdapter.Client","doc":"On connection failure, the Client GenServer:\n1. Detects the failure via process monitoring\n2. Cleans up the old Gun connection\n3. Opens a new Gun connection from the same GenServer process\n4. Maintains Gun message ownership continuity\n5. Preserves the same Client GenServer PID throughout\n\nThis ensures that integrated heartbeat functionality continues to work seamlessly\nacross reconnections without needing to track connection changes.\n\nThe Client GenServer handles all reconnection logic internally to maintain\nGun message ownership throughout the connection lifecycle.","ref":"WebsockexAdapter.Client.html#module-automatic-reconnection"},{"type":"module","title":"Core Functions - WebsockexAdapter.Client","doc":"- connect/2 - Establish connection\n- send_message/2 - Send messages  \n- close/1 - Close connection\n- subscribe/2 - Subscribe to channels\n- get_state/1 - Get connection state","ref":"WebsockexAdapter.Client.html#module-core-functions"},{"type":"module","title":"Configuration Options - WebsockexAdapter.Client","doc":"The `connect/2` function accepts all options from `WebsockexAdapter.Config`:\n\n    # Customize reconnection behavior\n    {:ok, client} = Client.connect(\"wss://example.com\",\n      retry_count: 5,              # Try reconnecting 5 times\n      retry_delay: 2000,           # Start with 2 second delay\n      max_backoff: 60_000,         # Cap backoff at 1 minute\n      reconnect_on_error: true     # Auto-reconnect on errors\n    )\n\n    # Disable auto-reconnection for critical operations\n    {:ok, client} = Client.connect(\"wss://example.com\",\n      reconnect_on_error: false\n    )\n\nSee `WebsockexAdapter.Config` for all available options.","ref":"WebsockexAdapter.Client.html#module-configuration-options"},{"type":"function","title":"WebsockexAdapter.Client.child_spec/1","doc":"Returns a child specification for starting a Client under a supervisor.","ref":"WebsockexAdapter.Client.html#child_spec/1"},{"type":"function","title":"Examples - WebsockexAdapter.Client.child_spec/1","doc":"# In your application's supervision tree\n    children = [\n      {WebsockexAdapter.Client, url: \"wss://example.com\", id: :my_client},\n      # Or with full configuration\n      {WebsockexAdapter.Client, [\n        url: \"wss://example.com\",\n        heartbeat_config: %{type: :deribit, interval: 30_000},\n        retry_count: 10\n      ]}\n    ]\n    \n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"WebsockexAdapter.Client.html#child_spec/1-examples"},{"type":"function","title":"WebsockexAdapter.Client.close/1","doc":"","ref":"WebsockexAdapter.Client.html#close/1"},{"type":"function","title":"WebsockexAdapter.Client.connect/2","doc":"","ref":"WebsockexAdapter.Client.html#connect/2"},{"type":"function","title":"WebsockexAdapter.Client.get_heartbeat_health/1","doc":"","ref":"WebsockexAdapter.Client.html#get_heartbeat_health/1"},{"type":"function","title":"WebsockexAdapter.Client.get_state/1","doc":"","ref":"WebsockexAdapter.Client.html#get_state/1"},{"type":"function","title":"WebsockexAdapter.Client.get_state_metrics/1","doc":"Gets detailed metrics about the client's internal state.\n\nReturns a map containing:\n- Data structure sizes (heartbeats, subscriptions, pending requests)\n- Memory usage information\n- Process statistics","ref":"WebsockexAdapter.Client.html#get_state_metrics/1"},{"type":"function","title":"WebsockexAdapter.Client.reconnect/1","doc":"","ref":"WebsockexAdapter.Client.html#reconnect/1"},{"type":"function","title":"WebsockexAdapter.Client.send_message/2","doc":"","ref":"WebsockexAdapter.Client.html#send_message/2"},{"type":"function","title":"WebsockexAdapter.Client.start_link/2","doc":"Starts a Client GenServer under a supervisor.\n\nThis function is designed to be called by a supervisor. For direct usage,\nprefer `connect/2` which provides better error handling and connection\nestablishment feedback.","ref":"WebsockexAdapter.Client.html#start_link/2"},{"type":"type","title":"WebsockexAdapter.Client.state/0","doc":"","ref":"WebsockexAdapter.Client.html#t:state/0"},{"type":"function","title":"WebsockexAdapter.Client.subscribe/2","doc":"","ref":"WebsockexAdapter.Client.html#subscribe/2"},{"type":"type","title":"WebsockexAdapter.Client.t/0","doc":"","ref":"WebsockexAdapter.Client.html#t:t/0"},{"type":"module","title":"WebsockexAdapter.ClientSupervisor","doc":"Optional supervisor for WebSocket client connections.\n\nProvides supervised client connections with automatic restart on failure.\nEach client runs under its own supervisor for isolation.","ref":"WebsockexAdapter.ClientSupervisor.html"},{"type":"module","title":"Adding to Your Supervision Tree - WebsockexAdapter.ClientSupervisor","doc":"# In your application supervisor\n    children = [\n      # Start the ClientSupervisor\n      WebsockexAdapter.ClientSupervisor,\n      # Your other children...\n    ]\n    \n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"WebsockexAdapter.ClientSupervisor.html#module-adding-to-your-supervision-tree"},{"type":"module","title":"Usage - WebsockexAdapter.ClientSupervisor","doc":"# After supervisor is started, create supervised connections\n    {:ok, client} = ClientSupervisor.start_client(\"wss://example.com\", \n      retry_count: 5,\n      heartbeat_config: %{type: :deribit, interval: 30_000}\n    )\n    \n    # The client will be automatically restarted on crashes\n    # with exponential backoff between restarts","ref":"WebsockexAdapter.ClientSupervisor.html#module-usage"},{"type":"function","title":"WebsockexAdapter.ClientSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexAdapter.ClientSupervisor.html#child_spec/1"},{"type":"function","title":"WebsockexAdapter.ClientSupervisor.list_clients/0","doc":"Lists all supervised client connections.","ref":"WebsockexAdapter.ClientSupervisor.html#list_clients/0"},{"type":"function","title":"WebsockexAdapter.ClientSupervisor.start_client/2","doc":"Starts a supervised WebSocket client.\n\nThe client will be automatically restarted on failure according to the\nsupervisor's restart strategy.","ref":"WebsockexAdapter.ClientSupervisor.html#start_client/2"},{"type":"function","title":"WebsockexAdapter.ClientSupervisor.start_link/1","doc":"Starts the client supervisor.","ref":"WebsockexAdapter.ClientSupervisor.html#start_link/1"},{"type":"function","title":"WebsockexAdapter.ClientSupervisor.stop_client/1","doc":"Gracefully stops a supervised client.","ref":"WebsockexAdapter.ClientSupervisor.html#stop_client/1"},{"type":"module","title":"WebsockexAdapter.Config","doc":"Configuration struct for WebSocket connections.","ref":"WebsockexAdapter.Config.html"},{"type":"module","title":"Options - WebsockexAdapter.Config","doc":"- `:url` - WebSocket URL (required, must be ws:// or wss://)\n- `:headers` - HTTP headers for the upgrade request (default: [])\n- `:timeout` - Connection timeout in milliseconds (default: 5000)\n- `:retry_count` - Maximum reconnection attempts (default: 3)\n- `:retry_delay` - Base delay for exponential backoff in ms (default: 1000)\n- `:heartbeat_interval` - Interval for heartbeat messages in ms (default: 30000)\n- `:max_backoff` - Maximum delay between reconnection attempts in ms (default: 30000)\n- `:reconnect_on_error` - Whether to auto-reconnect on connection errors (default: true)\n- `:restore_subscriptions` - Whether to restore subscriptions after reconnect (default: true)\n- `:request_timeout` - Timeout for correlated requests in ms (default: 30000)","ref":"WebsockexAdapter.Config.html#module-options"},{"type":"module","title":"Examples - WebsockexAdapter.Config","doc":"# Basic configuration\n    {:ok, config} = Config.new(\"wss://example.com\")\n\n    # Custom reconnection settings\n    {:ok, config} = Config.new(\"wss://example.com\",\n      retry_count: 5,\n      retry_delay: 2000,\n      max_backoff: 60_000,\n      reconnect_on_error: true\n    )\n\n    # Disable auto-reconnection\n    {:ok, config} = Config.new(\"wss://example.com\",\n      reconnect_on_error: false\n    )","ref":"WebsockexAdapter.Config.html#module-examples"},{"type":"function","title":"WebsockexAdapter.Config.new/2","doc":"Creates and validates a new configuration.","ref":"WebsockexAdapter.Config.html#new/2"},{"type":"function","title":"WebsockexAdapter.Config.new!/2","doc":"Creates and validates a new configuration, raising on error.","ref":"WebsockexAdapter.Config.html#new!/2"},{"type":"type","title":"WebsockexAdapter.Config.t/0","doc":"","ref":"WebsockexAdapter.Config.html#t:t/0"},{"type":"function","title":"WebsockexAdapter.Config.validate/1","doc":"Validates a configuration struct.","ref":"WebsockexAdapter.Config.html#validate/1"},{"type":"module","title":"WebsockexAdapter.ConnectionRegistry","doc":"ETS-based connection tracking without GenServer.","ref":"WebsockexAdapter.ConnectionRegistry.html"},{"type":"function","title":"WebsockexAdapter.ConnectionRegistry.cleanup_dead/1","doc":"Cleanup dead connection by PID.","ref":"WebsockexAdapter.ConnectionRegistry.html#cleanup_dead/1"},{"type":"function","title":"WebsockexAdapter.ConnectionRegistry.deregister/1","doc":"Deregister a connection by ID.","ref":"WebsockexAdapter.ConnectionRegistry.html#deregister/1"},{"type":"function","title":"WebsockexAdapter.ConnectionRegistry.get/1","doc":"Get connection info by ID.","ref":"WebsockexAdapter.ConnectionRegistry.html#get/1"},{"type":"function","title":"WebsockexAdapter.ConnectionRegistry.init/0","doc":"Initialize the connection registry ETS table.","ref":"WebsockexAdapter.ConnectionRegistry.html#init/0"},{"type":"function","title":"WebsockexAdapter.ConnectionRegistry.register/2","doc":"Register a connection with monitoring.","ref":"WebsockexAdapter.ConnectionRegistry.html#register/2"},{"type":"function","title":"WebsockexAdapter.ConnectionRegistry.shutdown/0","doc":"Cleanup all connections and destroy table.","ref":"WebsockexAdapter.ConnectionRegistry.html#shutdown/0"},{"type":"module","title":"WebsockexAdapter.ErrorHandler","doc":"Simple error handling for WebSocket connections.\n\nHandles common error scenarios:\n- Connection errors (network failures)\n- Protocol errors (malformed frames)\n- Authentication errors\n- Timeout errors\n\nPasses raw errors without wrapping to preserve original error information.","ref":"WebsockexAdapter.ErrorHandler.html"},{"type":"function","title":"WebsockexAdapter.ErrorHandler.categorize_error/1","doc":"Categorizes errors into recoverable vs non-recoverable types.\n\nReturns the raw error unchanged to preserve all original information.","ref":"WebsockexAdapter.ErrorHandler.html#categorize_error/1"},{"type":"function","title":"WebsockexAdapter.ErrorHandler.handle_error/1","doc":"Handles errors by returning appropriate actions.\n\nReturns either :reconnect or :stop based on error recoverability.","ref":"WebsockexAdapter.ErrorHandler.html#handle_error/1"},{"type":"function","title":"WebsockexAdapter.ErrorHandler.recoverable?/1","doc":"Determines if an error is recoverable through reconnection.","ref":"WebsockexAdapter.ErrorHandler.html#recoverable?/1"},{"type":"module","title":"WebsockexAdapter.Examples.AdapterSupervisor","doc":"Supervisor for WebSocket adapters with fault tolerance.\n\nThis supervisor manages adapter GenServers alongside the ClientSupervisor,\ncreating a robust supervision tree where:\n\n- ClientSupervisor manages Client GenServers\n- AdapterSupervisor manages adapter GenServers\n- Adapters monitor their Clients and handle reconnection","ref":"WebsockexAdapter.Examples.AdapterSupervisor.html"},{"type":"module","title":"Example - WebsockexAdapter.Examples.AdapterSupervisor","doc":"children = [\n      {WebsockexAdapter.ClientSupervisor, []},\n      {WebsockexAdapter.Examples.AdapterSupervisor, [\n        adapters: [\n          {DeribitGenServerAdapter, [\n            name: :deribit_main,\n            client_id: \"...\",\n            client_secret: \"...\"\n          ]},\n          {DeribitGenServerAdapter, [\n            name: :deribit_backup,\n            url: \"wss://www.deribit.com/ws/api/v2\",\n            client_id: \"...\",\n            client_secret: \"...\"\n          ]}\n        ]\n      ]}\n    ]\n    \n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"WebsockexAdapter.Examples.AdapterSupervisor.html#module-example"},{"type":"function","title":"WebsockexAdapter.Examples.AdapterSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexAdapter.Examples.AdapterSupervisor.html#child_spec/1"},{"type":"function","title":"WebsockexAdapter.Examples.AdapterSupervisor.start_link/1","doc":"","ref":"WebsockexAdapter.Examples.AdapterSupervisor.html#start_link/1"},{"type":"module","title":"WebsockexAdapter.Examples.BatchSubscriptionManager","doc":"Simple batch subscription manager for efficiently subscribing to multiple channels.\n\nPrevents overwhelming the API by batching subscription requests with configurable\nbatch size and delay between batches.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html"},{"type":"type","title":"WebsockexAdapter.Examples.BatchSubscriptionManager.batch_status/0","doc":"","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#t:batch_status/0"},{"type":"function","title":"WebsockexAdapter.Examples.BatchSubscriptionManager.cancel_batch/2","doc":"Cancels a batch subscription request.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#cancel_batch/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.BatchSubscriptionManager.cancel_batch/2","doc":"- `manager` - The batch manager process\n- `request_id` - The request ID to cancel","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#cancel_batch/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.BatchSubscriptionManager.cancel_batch/2","doc":"- `:ok` if cancelled successfully\n- `{:error, :not_found}` if request_id is invalid","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#cancel_batch/2-returns"},{"type":"function","title":"WebsockexAdapter.Examples.BatchSubscriptionManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#child_spec/1"},{"type":"function","title":"WebsockexAdapter.Examples.BatchSubscriptionManager.get_all_statuses/1","doc":"Gets the status of all batch requests.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#get_all_statuses/1"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.BatchSubscriptionManager.get_all_statuses/1","doc":"`{:ok, statuses}` where statuses is a map of request_id => status.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#get_all_statuses/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.BatchSubscriptionManager.get_status/2","doc":"Gets the status of a batch subscription request.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#get_status/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.BatchSubscriptionManager.get_status/2","doc":"- `manager` - The batch manager process\n- `request_id` - The request ID returned by subscribe_batch/2","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#get_status/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.BatchSubscriptionManager.get_status/2","doc":"- `{:ok, status}` with completed/pending/total counts\n- `{:error, :not_found}` if request_id is invalid","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#get_status/2-returns"},{"type":"function","title":"WebsockexAdapter.Examples.BatchSubscriptionManager.start_link/1","doc":"Starts the batch subscription manager.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#start_link/1"},{"type":"function","title":"Options - WebsockexAdapter.Examples.BatchSubscriptionManager.start_link/1","doc":"- `:adapter` - The Deribit adapter process (required)\n- `:batch_size` - Number of channels per batch (default: 10)\n- `:batch_delay` - Delay between batches in ms (default: 200)","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#start_link/1-options"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.BatchSubscriptionManager.start_link/1","doc":"`{:ok, pid}` on success.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#start_link/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.BatchSubscriptionManager.subscribe_batch/2","doc":"Subscribes to multiple channels in batches.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#subscribe_batch/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.BatchSubscriptionManager.subscribe_batch/2","doc":"- `manager` - The batch manager process\n- `channels` - List of channel names to subscribe to","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#subscribe_batch/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.BatchSubscriptionManager.subscribe_batch/2","doc":"`{:ok, request_id}` where request_id can be used to track progress.","ref":"WebsockexAdapter.Examples.BatchSubscriptionManager.html#subscribe_batch/2-returns"},{"type":"module","title":"WebsockexAdapter.Examples.DeribitAdapter","doc":"Simplified Deribit WebSocket API adapter.\n\nUses DeribitRpc for all RPC operations and provides\n5 essential functions for Deribit integration.","ref":"WebsockexAdapter.Examples.DeribitAdapter.html"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitAdapter.authenticate/1","doc":"Authenticate with Deribit using client credentials.","ref":"WebsockexAdapter.Examples.DeribitAdapter.html#authenticate/1"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitAdapter.connect/1","doc":"Connect to Deribit WebSocket API.\n\nOptions:\n- `:client_id` - Client ID for authentication\n- `:client_secret` - Client secret for authentication\n- `:url` - WebSocket URL (defaults to test.deribit.com)\n- `:handler` - Message handler function\n- `:heartbeat_interval` - Heartbeat interval in seconds (default: 30)","ref":"WebsockexAdapter.Examples.DeribitAdapter.html#connect/1"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitAdapter.send_request/3","doc":"Send a request to Deribit API using any supported method.","ref":"WebsockexAdapter.Examples.DeribitAdapter.html#send_request/3"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitAdapter.subscribe/2","doc":"Subscribe to Deribit channels.","ref":"WebsockexAdapter.Examples.DeribitAdapter.html#subscribe/2"},{"type":"type","title":"WebsockexAdapter.Examples.DeribitAdapter.t/0","doc":"","ref":"WebsockexAdapter.Examples.DeribitAdapter.html#t:t/0"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitAdapter.unsubscribe/2","doc":"Unsubscribe from Deribit channels.","ref":"WebsockexAdapter.Examples.DeribitAdapter.html#unsubscribe/2"},{"type":"module","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter","doc":"Production-ready supervised Deribit adapter with automatic reconnection.\n\nFeatures: Monitor WebSocket client, auto-reconnect, restore auth/subscriptions.\nOnly 5 public functions for clean API. See DeribitRpc for available methods.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter.authenticate/1","doc":"Authenticates with Deribit using configured credentials.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#authenticate/1"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitGenServerAdapter.authenticate/1","doc":"- `:ok` on successful authentication\n- `{:error, :not_connected}` if not connected\n- `{:error, :missing_credentials}` if credentials not configured","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#authenticate/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#child_spec/1"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter.get_state/1","doc":"Returns the current adapter state.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#get_state/1"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitGenServerAdapter.get_state/1","doc":"`{:ok, state}` with the internal state map.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#get_state/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter.send_request/3","doc":"Sends a JSON-RPC request to Deribit.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#send_request/3"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitGenServerAdapter.send_request/3","doc":"- `adapter` - The adapter GenServer\n- `method` - RPC method name\n- `params` - Method parameters (default: %{})","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#send_request/3-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitGenServerAdapter.send_request/3","doc":"The response from Deribit or `{:error, :not_connected}`.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#send_request/3-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter.start_link/1","doc":"Starts the Deribit adapter GenServer.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#start_link/1"},{"type":"function","title":"Options - WebsockexAdapter.Examples.DeribitGenServerAdapter.start_link/1","doc":"- `:name` - GenServer name (required)\n- `:client_id` - Deribit API client ID\n- `:client_secret` - Deribit API client secret\n- `:url` - WebSocket URL (defaults to test.deribit.com)\n- `:heartbeat_interval` - Heartbeat interval in seconds\n- `:handler` - Optional message handler module","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#start_link/1-options"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitGenServerAdapter.start_link/1","doc":"`{:ok, pid}` on success.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#start_link/1-returns"},{"type":"type","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter.state/0","doc":"","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#t:state/0"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitGenServerAdapter.subscribe/2","doc":"Subscribes to Deribit channels for real-time data.","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#subscribe/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitGenServerAdapter.subscribe/2","doc":"- `adapter` - The adapter GenServer\n- `channels` - List of channel names","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#subscribe/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitGenServerAdapter.subscribe/2","doc":"- `:ok` on successful subscription\n- `{:error, :not_connected}` if not connected","ref":"WebsockexAdapter.Examples.DeribitGenServerAdapter.html#subscribe/2-returns"},{"type":"module","title":"WebsockexAdapter.Examples.DeribitRpc","doc":"Shared Deribit JSON-RPC method definitions and request builders.\n\nThis module centralizes all Deribit RPC method definitions\nto avoid duplication across adapter examples.","ref":"WebsockexAdapter.Examples.DeribitRpc.html"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.auth_request/2","doc":"Builds an authentication request for Deribit API.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#auth_request/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.auth_request/2","doc":"- `client_id` - Your Deribit API client ID\n- `client_secret` - Your Deribit API client secret","ref":"WebsockexAdapter.Examples.DeribitRpc.html#auth_request/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.auth_request/2","doc":"A JSON-RPC request map for authentication.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#auth_request/2-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.build_request/2","doc":"Builds a JSON-RPC request for the given method and params.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#build_request/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.build_request/2","doc":"- `method` - The JSON-RPC method name\n- `params` - Optional parameters map (defaults to empty map)","ref":"WebsockexAdapter.Examples.DeribitRpc.html#build_request/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.build_request/2","doc":"A map containing the complete JSON-RPC request structure.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#build_request/2-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.buy/3","doc":"Creates a buy order for the specified instrument.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#buy/3"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.buy/3","doc":"- `instrument` - Instrument name (e.g., \"BTC-PERPETUAL\")\n- `amount` - Order amount in contracts\n- `opts` - Additional order options (type, price, etc.)","ref":"WebsockexAdapter.Examples.DeribitRpc.html#buy/3-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.buy/3","doc":"A JSON-RPC request map for buy order.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#buy/3-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.cancel/1","doc":"Cancels an existing order by ID.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#cancel/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.cancel/1","doc":"- `order_id` - The order ID to cancel","ref":"WebsockexAdapter.Examples.DeribitRpc.html#cancel/1-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.cancel/1","doc":"A JSON-RPC request map for order cancellation.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#cancel/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.get_instruments/1","doc":"Retrieves available trading instruments for a currency.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_instruments/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.get_instruments/1","doc":"- `currency` - Currency code (e.g., \"BTC\", \"ETH\")","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_instruments/1-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.get_instruments/1","doc":"A JSON-RPC request map for retrieving instruments.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_instruments/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.get_open_orders/1","doc":"Retrieves all open orders with optional filters.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_open_orders/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.get_open_orders/1","doc":"- `opts` - Optional filters (instrument, type, etc.)","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_open_orders/1-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.get_open_orders/1","doc":"A JSON-RPC request map for retrieving open orders.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_open_orders/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.get_order_book/2","doc":"Retrieves the order book for a specific instrument.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_order_book/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.get_order_book/2","doc":"- `instrument` - Instrument name (e.g., \"BTC-PERPETUAL\")\n- `depth` - Order book depth (default: 10)","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_order_book/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.get_order_book/2","doc":"A JSON-RPC request map for order book data.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#get_order_book/2-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.sell/3","doc":"Creates a sell order for the specified instrument.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#sell/3"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.sell/3","doc":"- `instrument` - Instrument name (e.g., \"BTC-PERPETUAL\")\n- `amount` - Order amount in contracts\n- `opts` - Additional order options (type, price, etc.)","ref":"WebsockexAdapter.Examples.DeribitRpc.html#sell/3-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.sell/3","doc":"A JSON-RPC request map for sell order.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#sell/3-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.set_heartbeat/1","doc":"Sets the heartbeat interval for the WebSocket connection.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#set_heartbeat/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.set_heartbeat/1","doc":"- `interval` - Heartbeat interval in seconds (default: 30)","ref":"WebsockexAdapter.Examples.DeribitRpc.html#set_heartbeat/1-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.set_heartbeat/1","doc":"A JSON-RPC request map for setting heartbeat.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#set_heartbeat/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.subscribe/1","doc":"Subscribes to one or more channels for real-time data.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#subscribe/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.subscribe/1","doc":"- `channels` - List of channel names to subscribe to","ref":"WebsockexAdapter.Examples.DeribitRpc.html#subscribe/1-parameters"},{"type":"function","title":"Example - WebsockexAdapter.Examples.DeribitRpc.subscribe/1","doc":"subscribe([\"book.BTC-PERPETUAL.raw\", \"ticker.ETH-PERPETUAL.raw\"])","ref":"WebsockexAdapter.Examples.DeribitRpc.html#subscribe/1-example"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.subscribe/1","doc":"A JSON-RPC request map for subscription.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#subscribe/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.test_request/0","doc":"Builds a test request to verify the connection is alive.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#test_request/0"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.test_request/0","doc":"A JSON-RPC request map for connection testing.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#test_request/0-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.ticker/1","doc":"Gets ticker data for a specific instrument.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#ticker/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.ticker/1","doc":"- `instrument` - Instrument name (e.g., \"BTC-PERPETUAL\")","ref":"WebsockexAdapter.Examples.DeribitRpc.html#ticker/1-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.ticker/1","doc":"A JSON-RPC request map for ticker data.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#ticker/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.DeribitRpc.unsubscribe/1","doc":"Unsubscribes from one or more channels.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#unsubscribe/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.DeribitRpc.unsubscribe/1","doc":"- `channels` - List of channel names to unsubscribe from","ref":"WebsockexAdapter.Examples.DeribitRpc.html#unsubscribe/1-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.DeribitRpc.unsubscribe/1","doc":"A JSON-RPC request map for unsubscription.","ref":"WebsockexAdapter.Examples.DeribitRpc.html#unsubscribe/1-returns"},{"type":"module","title":"WebsockexAdapter.Examples.Docs.BasicUsage","doc":"Basic usage examples from the documentation.\nThese examples demonstrate simple WebSocket connections and message handling.","ref":"WebsockexAdapter.Examples.Docs.BasicUsage.html"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.BasicUsage.custom_headers_example/1","doc":"Connection with custom headers example.\n\nShows how to connect with authorization headers and other custom headers.","ref":"WebsockexAdapter.Examples.Docs.BasicUsage.html#custom_headers_example/1"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.BasicUsage.echo_server_example/0","doc":"Simple echo server connection example.\n\nConnects to an echo WebSocket server, sends a message, and receives the echo.\nThe default handler automatically sends messages to the calling process.","ref":"WebsockexAdapter.Examples.Docs.BasicUsage.html#echo_server_example/0"},{"type":"module","title":"WebsockexAdapter.Examples.Docs.ErrorHandling","doc":"Error handling and retry patterns from Examples.md","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.ErrorHandling.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#child_spec/1"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.ErrorHandling.get_state/0","doc":"Returns the current state of the error handler.","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#get_state/0"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.Docs.ErrorHandling.get_state/0","doc":"The internal state map including connection status and retry count.","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#get_state/0-returns"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.ErrorHandling.send_message/1","doc":"Sends a message through the WebSocket connection.","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#send_message/1"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.Docs.ErrorHandling.send_message/1","doc":"- `message` - Message to send (will be JSON encoded)","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#send_message/1-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.Docs.ErrorHandling.send_message/1","doc":"- `:ok` on success\n- `{:error, :not_connected}` if not connected","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#send_message/1-returns"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.ErrorHandling.start_link/2","doc":"Starts a GenServer that manages a WebSocket connection with automatic retry.","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#start_link/2"},{"type":"function","title":"Parameters - WebsockexAdapter.Examples.Docs.ErrorHandling.start_link/2","doc":"- `url` - WebSocket URL to connect to\n- `opts` - Connection options","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#start_link/2-parameters"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.Docs.ErrorHandling.start_link/2","doc":"`{:ok, pid}` on success or `{:error, reason}` on failure.","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#start_link/2-returns"},{"type":"type","title":"WebsockexAdapter.Examples.Docs.ErrorHandling.state/0","doc":"","ref":"WebsockexAdapter.Examples.Docs.ErrorHandling.html#t:state/0"},{"type":"module","title":"WebsockexAdapter.Examples.Docs.JsonRpcClient","doc":"JSON-RPC client example from documentation.\n\nDemonstrates how to make JSON-RPC calls over WebSocket connections.\nThe WebsockexAdapter.Client automatically handles request/response \ncorrelation for JSON-RPC messages - no manual correlation needed!","ref":"WebsockexAdapter.Examples.Docs.JsonRpcClient.html"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.JsonRpcClient.call_method/4","doc":"Makes a synchronous JSON-RPC call.\n\nThe Client automatically handles JSON-RPC correlation - when sending a message\nwith an \"id\" field, it tracks the request and returns the correlated response.","ref":"WebsockexAdapter.Examples.Docs.JsonRpcClient.html#call_method/4"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.JsonRpcClient.cast_method/3","doc":"Makes an async JSON-RPC call without waiting for response.","ref":"WebsockexAdapter.Examples.Docs.JsonRpcClient.html#cast_method/3"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.JsonRpcClient.echo/1","doc":"Echo test method","ref":"WebsockexAdapter.Examples.Docs.JsonRpcClient.html#echo/1"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.JsonRpcClient.get_balance/1","doc":"Get account balance","ref":"WebsockexAdapter.Examples.Docs.JsonRpcClient.html#get_balance/1"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.JsonRpcClient.get_server_time/1","doc":"Get server timestamp","ref":"WebsockexAdapter.Examples.Docs.JsonRpcClient.html#get_server_time/1"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.JsonRpcClient.handle_message/1","doc":"Processes incoming WebSocket messages and returns JSON-RPC responses.","ref":"WebsockexAdapter.Examples.Docs.JsonRpcClient.html#handle_message/1"},{"type":"module","title":"WebsockexAdapter.Examples.Docs.SubscriptionManagement","doc":"Examples of subscription management with WebSocket connections.","ref":"WebsockexAdapter.Examples.Docs.SubscriptionManagement.html"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.SubscriptionManagement.handle_market_data/1","doc":"Process market data updates with pattern matching.","ref":"WebsockexAdapter.Examples.Docs.SubscriptionManagement.html#handle_market_data/1"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.SubscriptionManagement.multi_channel_subscription/0","doc":"Subscribe to multiple data streams using echo server.","ref":"WebsockexAdapter.Examples.Docs.SubscriptionManagement.html#multi_channel_subscription/0"},{"type":"function","title":"WebsockexAdapter.Examples.Docs.SubscriptionManagement.subscription_loop/3","doc":"Subscribe and process messages in a loop.","ref":"WebsockexAdapter.Examples.Docs.SubscriptionManagement.html#subscription_loop/3"},{"type":"module","title":"WebsockexAdapter.Examples.PlatformAdapterTemplate","doc":"Minimal template for creating platform-specific WebSocket adapters.","ref":"WebsockexAdapter.Examples.PlatformAdapterTemplate.html"},{"type":"module","title":"Extension Points - WebsockexAdapter.Examples.PlatformAdapterTemplate","doc":"- `handle_message/2` - Process platform-specific messages\n- `format_subscription/1` - Convert channels to platform format\n- `authenticate/2` - Implement platform authentication","ref":"WebsockexAdapter.Examples.PlatformAdapterTemplate.html#module-extension-points"},{"type":"function","title":"WebsockexAdapter.Examples.PlatformAdapterTemplate.authenticate/2","doc":"Authenticate with platform credentials","ref":"WebsockexAdapter.Examples.PlatformAdapterTemplate.html#authenticate/2"},{"type":"function","title":"WebsockexAdapter.Examples.PlatformAdapterTemplate.connect/2","doc":"Connect to platform WebSocket endpoint","ref":"WebsockexAdapter.Examples.PlatformAdapterTemplate.html#connect/2"},{"type":"function","title":"WebsockexAdapter.Examples.PlatformAdapterTemplate.handle_message/2","doc":"Handle incoming platform messages","ref":"WebsockexAdapter.Examples.PlatformAdapterTemplate.html#handle_message/2"},{"type":"function","title":"WebsockexAdapter.Examples.PlatformAdapterTemplate.request/3","doc":"Send platform request","ref":"WebsockexAdapter.Examples.PlatformAdapterTemplate.html#request/3"},{"type":"function","title":"WebsockexAdapter.Examples.PlatformAdapterTemplate.subscribe/2","doc":"Subscribe to platform channels","ref":"WebsockexAdapter.Examples.PlatformAdapterTemplate.html#subscribe/2"},{"type":"module","title":"WebsockexAdapter.Examples.SupervisedClient","doc":"Basic WebSocket client supervision example.\n\nShows minimal setup for supervised connections. For advanced patterns,\nsee the supervision documentation.","ref":"WebsockexAdapter.Examples.SupervisedClient.html"},{"type":"function","title":"WebsockexAdapter.Examples.SupervisedClient.list_connections/0","doc":"Lists all supervised connections.","ref":"WebsockexAdapter.Examples.SupervisedClient.html#list_connections/0"},{"type":"function","title":"WebsockexAdapter.Examples.SupervisedClient.start_connection/2","doc":"Starts a supervised WebSocket connection.","ref":"WebsockexAdapter.Examples.SupervisedClient.html#start_connection/2"},{"type":"function","title":"Example - WebsockexAdapter.Examples.SupervisedClient.start_connection/2","doc":"{:ok, client} = SupervisedClient.start_connection(\"wss://echo.websocket.org\")","ref":"WebsockexAdapter.Examples.SupervisedClient.html#start_connection/2-example"},{"type":"function","title":"WebsockexAdapter.Examples.SupervisedClient.start_multiple/1","doc":"Starts multiple supervised connections.","ref":"WebsockexAdapter.Examples.SupervisedClient.html#start_multiple/1"},{"type":"function","title":"Example - WebsockexAdapter.Examples.SupervisedClient.start_multiple/1","doc":"clients = SupervisedClient.start_multiple([\n      {\"wss://api1.example.com\", retry_count: 5},\n      {\"wss://api2.example.com\", heartbeat_interval: 20_000}\n    ])","ref":"WebsockexAdapter.Examples.SupervisedClient.html#start_multiple/1-example"},{"type":"function","title":"WebsockexAdapter.Examples.SupervisedClient.stop_connection/1","doc":"Stops a supervised connection.","ref":"WebsockexAdapter.Examples.SupervisedClient.html#stop_connection/1"},{"type":"module","title":"WebsockexAdapter.Examples.UsagePatterns","doc":"Examples of different WebSocket client usage patterns.\nThis module demonstrates three ways to use WebsockexAdapter:\n1. Direct connection (no supervision)\n2. Using ClientSupervisor\n3. Direct supervision in your app","ref":"WebsockexAdapter.Examples.UsagePatterns.html"},{"type":"function","title":"WebsockexAdapter.Examples.UsagePatterns.client_supervisor_example/0","doc":"Pattern 2: Using ClientSupervisor for automatic restarts.\n\nBest for:\n- Production systems with multiple connections\n- Dynamic connection management\n- When you need a connection pool\n\nNote: You must add ClientSupervisor to your supervision tree first!","ref":"WebsockexAdapter.Examples.UsagePatterns.html#client_supervisor_example/0"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.UsagePatterns.client_supervisor_example/0","doc":"`:ok` after demonstrating the supervisor pattern.","ref":"WebsockexAdapter.Examples.UsagePatterns.html#client_supervisor_example/0-returns"},{"type":"function","title":"WebsockexAdapter.Examples.UsagePatterns.direct_connection_example/0","doc":"Pattern 1: Direct connection without supervision.\n\nBest for:\n- Development and testing\n- Short-lived connections\n- Scripts and one-off tasks","ref":"WebsockexAdapter.Examples.UsagePatterns.html#direct_connection_example/0"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.UsagePatterns.direct_connection_example/0","doc":"`:ok` after demonstrating the connection pattern.","ref":"WebsockexAdapter.Examples.UsagePatterns.html#direct_connection_example/0-returns"},{"type":"function","title":"WebsockexAdapter.Examples.UsagePatterns.supervised_client_spec/0","doc":"Pattern 3: Direct supervision in your application.\n\nBest for:\n- Fixed set of connections\n- When each connection has a specific role\n- Simple production deployments\n\nAdd this to your application supervisor:\n\n    children = [\n      {WebsockexAdapter.Client, [\n        url: \"wss://test.deribit.com/ws/api/v2\",\n        id: :deribit_client,\n        heartbeat_config: %{type: :deribit, interval: 30_000}\n      ]}\n    ]","ref":"WebsockexAdapter.Examples.UsagePatterns.html#supervised_client_spec/0"},{"type":"function","title":"Returns - WebsockexAdapter.Examples.UsagePatterns.supervised_client_spec/0","doc":"A child specification tuple for use with Supervisor.","ref":"WebsockexAdapter.Examples.UsagePatterns.html#supervised_client_spec/0-returns"},{"type":"module","title":"WebsockexAdapter.Examples.UsagePatterns.ExampleApp","doc":"Example application module demonstrating all WebSocket connection patterns.\n\nThis module shows how to set up both dynamic (ClientSupervisor) and\nstatic (direct child specs) WebSocket connections in your supervision tree.","ref":"WebsockexAdapter.Examples.UsagePatterns.ExampleApp.html"},{"type":"module","title":"WebsockexAdapter.Frame","doc":"WebSocket frame encoding and decoding utilities.","ref":"WebsockexAdapter.Frame.html"},{"type":"function","title":"WebsockexAdapter.Frame.binary/1","doc":"Encode binary message as WebSocket frame.","ref":"WebsockexAdapter.Frame.html#binary/1"},{"type":"function","title":"WebsockexAdapter.Frame.decode/1","doc":"Decode incoming WebSocket frame.\nHandles both Gun WebSocket format {:ws, type, data} and direct frame format {:type, data}.","ref":"WebsockexAdapter.Frame.html#decode/1"},{"type":"type","title":"WebsockexAdapter.Frame.frame/0","doc":"","ref":"WebsockexAdapter.Frame.html#t:frame/0"},{"type":"type","title":"WebsockexAdapter.Frame.frame_type/0","doc":"","ref":"WebsockexAdapter.Frame.html#t:frame_type/0"},{"type":"function","title":"WebsockexAdapter.Frame.ping/0","doc":"Create ping frame.","ref":"WebsockexAdapter.Frame.html#ping/0"},{"type":"function","title":"WebsockexAdapter.Frame.pong/1","doc":"Create pong frame with payload.","ref":"WebsockexAdapter.Frame.html#pong/1"},{"type":"function","title":"WebsockexAdapter.Frame.text/1","doc":"Encode text message as WebSocket frame.","ref":"WebsockexAdapter.Frame.html#text/1"},{"type":"module","title":"WebsockexAdapter.Helpers.Deribit","doc":"Helper functions for Deribit-specific WebSocket operations.","ref":"WebsockexAdapter.Helpers.Deribit.html"},{"type":"function","title":"WebsockexAdapter.Helpers.Deribit.handle_heartbeat/2","doc":"Handles Deribit test_request heartbeat messages.","ref":"WebsockexAdapter.Helpers.Deribit.html#handle_heartbeat/2"},{"type":"function","title":"WebsockexAdapter.Helpers.Deribit.send_heartbeat/1","doc":"Sends Deribit heartbeat ping message.","ref":"WebsockexAdapter.Helpers.Deribit.html#send_heartbeat/1"},{"type":"module","title":"WebsockexAdapter.JsonRpc","doc":"JSON-RPC 2.0 request builder and response matcher.\n\nSimple API builder for WebSocket APIs using JSON-RPC 2.0 protocol.\nGenerates request functions with automatic ID tracking and correlation.","ref":"WebsockexAdapter.JsonRpc.html"},{"type":"function","title":"WebsockexAdapter.JsonRpc.build_request/2","doc":"Builds a JSON-RPC 2.0 request with unique ID.","ref":"WebsockexAdapter.JsonRpc.html#build_request/2"},{"type":"function","title":"Examples - WebsockexAdapter.JsonRpc.build_request/2","doc":"iex> {:ok, request} = JsonRpc.build_request(\"public/auth\", %{grant_type: \"client_credentials\"})\n    iex> request[\"method\"]\n    \"public/auth\"","ref":"WebsockexAdapter.JsonRpc.html#build_request/2-examples"},{"type":"macro","title":"WebsockexAdapter.JsonRpc.defrpc/3","doc":"Generates RPC method functions with automatic request building.","ref":"WebsockexAdapter.JsonRpc.html#defrpc/3"},{"type":"macro","title":"Examples - WebsockexAdapter.JsonRpc.defrpc/3","doc":"defmodule MyApi do\n      use WebsockexAdapter.JsonRpc\n\n      defrpc :authenticate, \"public/auth\"\n      defrpc :subscribe, \"public/subscribe\"\n      defrpc :get_order_book, \"public/get_order_book\"\n    end","ref":"WebsockexAdapter.JsonRpc.html#defrpc/3-examples"},{"type":"function","title":"WebsockexAdapter.JsonRpc.match_response/1","doc":"Matches a JSON-RPC response to determine if it's a result or error.\n\nReturns:\n- `{:ok, result}` for successful responses\n- `{:error, {code, message}}` for JSON-RPC errors\n- `{:notification, method, params}` for notifications","ref":"WebsockexAdapter.JsonRpc.html#match_response/1"},{"type":"module","title":"WebsockexAdapter.MessageHandler","doc":"Message handling utilities for WebSocket connections.\n\n- Parse incoming WebSocket frames and Gun messages\n- Route messages to user-provided handler functions  \n- Handle control frames (ping/pong) automatically\n- Process WebSocket upgrade responses","ref":"WebsockexAdapter.MessageHandler.html"},{"type":"function","title":"WebsockexAdapter.MessageHandler.create_handler/1","doc":"Create a callback function for handling specific message types.","ref":"WebsockexAdapter.MessageHandler.html#create_handler/1"},{"type":"function","title":"WebsockexAdapter.MessageHandler.default_handler/1","doc":"Default message handler that simply logs messages.","ref":"WebsockexAdapter.MessageHandler.html#default_handler/1"},{"type":"function","title":"WebsockexAdapter.MessageHandler.handle_control_frame/3","doc":"Handle WebSocket control frames automatically.\nReturns :handled for control frames, :not_control for data frames.","ref":"WebsockexAdapter.MessageHandler.html#handle_control_frame/3"},{"type":"function","title":"WebsockexAdapter.MessageHandler.handle_message/2","doc":"Handle incoming Gun messages and WebSocket frames.\nRoutes messages to appropriate handler function.","ref":"WebsockexAdapter.MessageHandler.html#handle_message/2"},{"type":"module","title":"WebsockexAdapter.RateLimiter","doc":"Token bucket rate limiter for WebSocket API calls.\n\nPrevents rate limit violations with configurable cost functions\nsupporting credit-based (Deribit), weight-based (Binance), and\nsimple rate limit (Coinbase) patterns through single algorithm.","ref":"WebsockexAdapter.RateLimiter.html"},{"type":"function","title":"WebsockexAdapter.RateLimiter.binance_cost/1","doc":"Binance weight-based cost function.","ref":"WebsockexAdapter.RateLimiter.html#binance_cost/1"},{"type":"type","title":"WebsockexAdapter.RateLimiter.config/0","doc":"","ref":"WebsockexAdapter.RateLimiter.html#t:config/0"},{"type":"function","title":"WebsockexAdapter.RateLimiter.consume/2","doc":"Attempts to consume tokens for a request.\n\nReturns :ok if tokens available, queues request if not.","ref":"WebsockexAdapter.RateLimiter.html#consume/2"},{"type":"function","title":"WebsockexAdapter.RateLimiter.deribit_cost/1","doc":"Deribit credit-based cost function.","ref":"WebsockexAdapter.RateLimiter.html#deribit_cost/1"},{"type":"function","title":"WebsockexAdapter.RateLimiter.init/2","doc":"Initializes rate limiter with configuration.\n\nCreates ETS table for state storage and schedules refill timer.","ref":"WebsockexAdapter.RateLimiter.html#init/2"},{"type":"function","title":"WebsockexAdapter.RateLimiter.refill/1","doc":"Refills tokens at configured rate.\n\nCalled by timer process at refill intervals.","ref":"WebsockexAdapter.RateLimiter.html#refill/1"},{"type":"function","title":"WebsockexAdapter.RateLimiter.simple_cost/1","doc":"Simple cost function for fixed-rate exchanges.","ref":"WebsockexAdapter.RateLimiter.html#simple_cost/1"},{"type":"type","title":"WebsockexAdapter.RateLimiter.state/0","doc":"","ref":"WebsockexAdapter.RateLimiter.html#t:state/0"},{"type":"function","title":"WebsockexAdapter.RateLimiter.status/1","doc":"Returns current token count and queue size.","ref":"WebsockexAdapter.RateLimiter.html#status/1"},{"type":"module","title":"WebsockexAdapter.Reconnection","doc":"Internal reconnection helper for Client GenServer.\n\nThis module provides reconnection logic that runs within the Client GenServer\nprocess to maintain Gun message ownership. It handles:\n\n- Connection establishment with retry logic\n- Exponential backoff calculations\n- Subscription restoration after reconnection","ref":"WebsockexAdapter.Reconnection.html"},{"type":"module","title":"Architecture - WebsockexAdapter.Reconnection","doc":"This module is called by the Client GenServer during its handle_continue\nand handle_info callbacks. All functions run in the Client GenServer process\nto ensure the new Gun connection sends messages to the correct process.","ref":"WebsockexAdapter.Reconnection.html#module-architecture"},{"type":"module","title":"Not for External Use - WebsockexAdapter.Reconnection","doc":"This module is internal to WebsockexAdapter. External code should use\n`WebsockexAdapter.Client.connect/2` which handles initial connection attempts\nand automatic reconnection.","ref":"WebsockexAdapter.Reconnection.html#module-not-for-external-use"},{"type":"function","title":"WebsockexAdapter.Reconnection.calculate_backoff/3","doc":"Calculate exponential backoff delay for reconnection attempts.","ref":"WebsockexAdapter.Reconnection.html#calculate_backoff/3"},{"type":"function","title":"Examples - WebsockexAdapter.Reconnection.calculate_backoff/3","doc":"iex> calculate_backoff(0, 1000)\n    1000\n\n    iex> calculate_backoff(1, 1000)\n    2000\n\n    iex> calculate_backoff(5, 1000, 30000)\n    30000  # Capped at max_backoff","ref":"WebsockexAdapter.Reconnection.html#calculate_backoff/3-examples"},{"type":"function","title":"WebsockexAdapter.Reconnection.establish_connection/1","doc":"Attempt to establish a Gun connection with the given configuration.\n\nThis function must be called from within the Client GenServer process\nto ensure Gun sends messages to the correct process.","ref":"WebsockexAdapter.Reconnection.html#establish_connection/1"},{"type":"function","title":"WebsockexAdapter.Reconnection.max_retries_exceeded?/2","doc":"Check if maximum retry attempts have been exceeded.","ref":"WebsockexAdapter.Reconnection.html#max_retries_exceeded?/2"},{"type":"function","title":"WebsockexAdapter.Reconnection.restore_subscriptions/3","doc":"Restore subscriptions after successful reconnection.\n\nThis should be called after the WebSocket upgrade is complete and the\nconnection is ready to receive subscription messages.","ref":"WebsockexAdapter.Reconnection.html#restore_subscriptions/3"},{"type":"function","title":"WebsockexAdapter.Reconnection.should_reconnect?/1","doc":"Determine if a connection error should trigger reconnection.\n\nReturns true for recoverable errors like network issues, false for\nunrecoverable errors like invalid credentials.","ref":"WebsockexAdapter.Reconnection.html#should_reconnect?/1"},{"type":"extras","title":"WebsockexAdapter","doc":"# WebsockexAdapter\n\n[![Hex.pm](https://img.shields.io/hexpm/v/websockex_adapter.svg)](https://hex.pm/packages/websockex_adapter)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-purple.svg)](https://hexdocs.pm/websockex_adapter)\n[![License](https://img.shields.io/hexpm/l/websockex_adapter.svg)](https://github.com/ZenHive/websockex_adapter/blob/main/LICENSE)\n\nA robust WebSocket client library for Elixir, built on Gun transport for production-grade reliability. Designed for financial APIs with automatic reconnection, comprehensive error handling, and real-world testing.","ref":"readme.html"},{"type":"extras","title":"Features - WebsockexAdapter","doc":"- **Gun Transport** - Battle-tested HTTP/2 and WebSocket client\n- **Automatic Reconnection** - Exponential backoff with state preservation\n- **Financial-Grade Reliability** - Built for high-frequency trading systems\n- **Platform Adapters** - Ready-to-use Deribit integration, extensible for others\n- **Real API Testing** - No mocks, tested against live systems\n- **Simple API** - Only 5 public functions to learn\n- **Comprehensive Error Handling** - Categorized errors with recovery strategies\n- **Rate Limiting** - Configurable token bucket algorithm\n- **JSON-RPC 2.0** - Full protocol support with correlation tracking","ref":"readme.html#features"},{"type":"extras","title":"Installation - WebsockexAdapter","doc":"Add `websockex_adapter` to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:websockex_adapter, \"~> 0.1.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - WebsockexAdapter","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Basic Connection - WebsockexAdapter","doc":"```elixir\n# Connect to a WebSocket endpoint\n{:ok, client} = WebsockexAdapter.Client.connect(\"wss://echo.websocket.org\", [\n  timeout: 5000,\n  heartbeat_interval: 30000\n])\n\n# Send a message\n{:ok, _} = WebsockexAdapter.Client.send_message(client, \"Hello, WebSocket!\")\n\n# Receive messages in your process\nreceive do\n  {:websocket_message, message} -> IO.inspect(message)\nend\n\n# Close the connection\n:ok = WebsockexAdapter.Client.close(client)\n```\n\nFor more detailed examples, see our working examples with fully tested implementations:\n- **Basic Usage** - Connection management and messaging\n- **Error Handling** - Robust error recovery patterns  \n- **JSON-RPC Client** - JSON-RPC 2.0 protocol usage\n- **Subscription Management** - Channel subscription patterns\n\nSee the [Examples Guide](https://hexdocs.pm/websockex_adapter/Examples.html) for complete code samples and usage patterns.","ref":"readme.html#basic-connection"},{"type":"extras","title":"Deribit Integration - WebsockexAdapter","doc":"```elixir\n# Configure Deribit credentials\nconfig = %{\n  client_id: System.get_env(\"DERIBIT_CLIENT_ID\"),\n  client_secret: System.get_env(\"DERIBIT_CLIENT_SECRET\"),\n  test_mode: true\n}\n\n# Start the supervised adapter\n{:ok, adapter} = WebsockexAdapter.Examples.DeribitGenServerAdapter.start_link(config)\n\n# Subscribe to market data\n{:ok, _} = WebsockexAdapter.Examples.DeribitGenServerAdapter.subscribe(\n  adapter,\n  [\"book.BTC-PERPETUAL.raw\", \"trades.BTC-PERPETUAL.raw\"]\n)\n\n# Place an order\n{:ok, order} = WebsockexAdapter.Examples.DeribitGenServerAdapter.place_order(adapter, %{\n  instrument_name: \"BTC-PERPETUAL\",\n  amount: 10,\n  type: \"limit\",\n  price: 50000,\n  direction: \"buy\"\n})\n```","ref":"readme.html#deribit-integration"},{"type":"extras","title":"Architecture - WebsockexAdapter","doc":"WebsockexAdapter follows a modular architecture with clear separation of concerns:\n\n```\nWebsockexAdapter.Client         # Main client interface\nWebsockexAdapter.Config         # Configuration management\nWebsockexAdapter.Frame          # WebSocket frame handling\nWebsockexAdapter.Reconnection   # Automatic reconnection logic\nWebsockexAdapter.MessageHandler # Message parsing and routing\nWebsockexAdapter.ErrorHandler   # Error categorization\nWebsockexAdapter.RateLimiter   # API rate limiting\nWebsockexAdapter.JsonRpc       # JSON-RPC 2.0 protocol\n```","ref":"readme.html#architecture"},{"type":"extras","title":"Platform Integration - WebsockexAdapter","doc":"The library includes a complete Deribit adapter as a reference implementation. To integrate with other platforms:\n\n1. Create an adapter module following the Deribit pattern\n2. Implement platform-specific authentication\n3. Handle platform message formats\n4. Add comprehensive tests against the real API\n\nSee `lib/websockex_adapter/examples/deribit_adapter.ex` for a complete example.","ref":"readme.html#platform-integration"},{"type":"extras","title":"Configuration Options - WebsockexAdapter","doc":"- `url` - WebSocket endpoint URL\n- `headers` - Custom headers for connection\n- `timeout` - Connection timeout in milliseconds (default: 5000)\n- `retry_count` - Maximum reconnection attempts (default: 3)\n- `retry_delay` - Initial retry delay in milliseconds (default: 1000)\n- `heartbeat_interval` - Ping interval in milliseconds (default: 30000)\n- `reconnect_on_error` - Enable automatic reconnection (default: true)","ref":"readme.html#configuration-options"},{"type":"extras","title":"Testing Philosophy - WebsockexAdapter","doc":"This library uses **real API testing exclusively**. No mocks or stubs - every test runs against actual WebSocket endpoints or local test servers. This ensures the library handles real-world conditions including network latency, connection drops, and API quirks.\n\n```bash\n# Run all tests\nmix test\n\n# Run with coverage\nmix test --cover\n\n# Run quality checks\nmix check\n```","ref":"readme.html#testing-philosophy"},{"type":"extras","title":"Contributing - WebsockexAdapter","doc":"1. Fork the repository\n2. Create your feature branch (`git checkout -b feature/amazing-feature`)\n3. Write tests using real APIs (no mocks)\n4. Ensure all quality checks pass (`mix check`)\n5. Commit your changes\n6. Push to the branch\n7. Open a Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"Development Commands - WebsockexAdapter","doc":"```bash\nmix compile           # Compile the project\nmix test             # Run test suite\nmix lint             # Run Credo analysis\nmix typecheck        # Run Dialyzer\nmix docs             # Generate documentation\nmix check            # Run all quality checks\n```","ref":"readme.html#development-commands"},{"type":"extras","title":"License - WebsockexAdapter","doc":"This project is licensed under the MIT License.","ref":"readme.html#license"},{"type":"extras","title":"Links - WebsockexAdapter","doc":"- [Documentation](https://hexdocs.pm/websockex_adapter)\n- [Hex Package](https://hex.pm/packages/websockex_adapter)\n- [GitHub Repository](https://github.com/ZenHive/websockex_adapter)","ref":"readme.html#links"},{"type":"extras","title":"Acknowledgments - WebsockexAdapter","doc":"Built with d\u000F for the Elixir community by [ZenHive](https://github.com/ZenHive).","ref":"readme.html#acknowledgments"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.1.1] - 2025-05-24","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release of WebsockexAdapter\n- Core WebSocket client implementation with Gun transport\n- Automatic reconnection with exponential backoff\n- Comprehensive error handling and categorization\n- JSON-RPC 2.0 protocol support\n- Request/response correlation manager\n- Configurable token bucket rate limiter\n- Integrated heartbeat/keepalive functionality\n- Fault-tolerant adapter architecture\n- Production-ready Deribit exchange integration\n- Connection registry for multi-connection management\n- Message handler with routing capabilities\n- WebSocket frame encoding/decoding\n- Telemetry events for monitoring\n- Comprehensive test suite using real APIs (no mocks)\n- Full documentation with examples","ref":"changelog.html#added"},{"type":"extras","title":"Features - Changelog","doc":"- Simple 5-function public API\n- Financial-grade reliability for trading systems\n- Platform-agnostic design with adapter pattern\n- Real-world tested against live WebSocket endpoints\n- Strict code quality standards (max 5 functions per module, 15 lines per function)\n\n[Unreleased]: https://github.com/ZenHive/websockex_adapter/compare/v0.1.1...HEAD\n[0.1.1]: https://github.com/ZenHive/websockex_adapter/releases/tag/v0.1.1","ref":"changelog.html#features"},{"type":"extras","title":"WebsockexAdapter Examples","doc":"# WebsockexAdapter Examples\n\nThis guide provides practical examples of using WebsockexAdapter in various scenarios. All examples are fully tested and available in the `lib/websockex_adapter/examples/` directory.","ref":"examples.html"},{"type":"extras","title":"Working Examples - WebsockexAdapter Examples","doc":"All examples have comprehensive test coverage and demonstrate real-world usage patterns:","ref":"examples.html#working-examples"},{"type":"extras","title":"Core Examples (in examples/docs/) - WebsockexAdapter Examples","doc":"- **[Basic Usage](../lib/websockex_adapter/examples/docs/basic_usage.ex)** - Basic connection and messaging patterns ([tests](../test/websockex_adapter/examples/basic_usage_test.exs))\n- **[Error Handling](../lib/websockex_adapter/examples/docs/error_handling.ex)** - Robust error recovery patterns ([tests](../test/websockex_adapter/examples/error_handling_test.exs))\n- **[JSON-RPC Client](../lib/websockex_adapter/examples/docs/json_rpc_client.ex)** - JSON-RPC 2.0 protocol usage\n- **[Subscription Management](../lib/websockex_adapter/examples/docs/subscription_management.ex)** - Channel subscription patterns ([tests](../test/websockex_adapter/examples/subscription_management_test.exs))","ref":"examples.html#core-examples-in-examples-docs"},{"type":"extras","title":"Platform Adapters - WebsockexAdapter Examples","doc":"- **Platform Adapter Template** - Template for creating platform-specific adapters (see platform_adapter_template.ex)\n- **Deribit Integration** - Complete examples moved to market_maker project for better separation of concerns","ref":"examples.html#platform-adapters"},{"type":"extras","title":"Architecture Examples - WebsockexAdapter Examples","doc":"- **[Adapter Supervisor](../lib/websockex_adapter/examples/adapter_supervisor.ex)** - Fault-tolerant supervision patterns\n- **[Supervised Client](../lib/websockex_adapter/examples/supervised_client.ex)** - Client supervision with restart strategies\n- **[Usage Patterns](../lib/websockex_adapter/examples/usage_patterns.ex)** - Common WebSocket patterns and best practices","ref":"examples.html#architecture-examples"},{"type":"extras","title":"Running the Examples - WebsockexAdapter Examples","doc":"","ref":"examples.html#running-the-examples"},{"type":"extras","title":"Basic Connection Example - WebsockexAdapter Examples","doc":"```bash\n# Run the basic usage example test\nmix test test/websockex_adapter/examples/basic_usage_test.exs\n\n# Or run it in IEx\niex -S mix\n```\n\n```elixir\n# In IEx, try the basic usage example\nalias WebsockexAdapter.Examples.Docs.BasicUsage\n\n# Echo server example\n{:ok, result} = BasicUsage.echo_example()\n\n# Custom headers example\n{:ok, client} = BasicUsage.connect_with_headers(\"wss://echo.websocket.org\", \"Bearer token123\")\n```","ref":"examples.html#basic-connection-example"},{"type":"extras","title":"Platform Integration Example - WebsockexAdapter Examples","doc":"For comprehensive platform integration examples (including Deribit), see the `market_maker` project which demonstrates:\n- Authentication flows\n- Supervised adapters with automatic recovery\n- Market data subscriptions\n- Trading operations\n- Error handling and reconnection\n\nThe WebsockexAdapter library provides the infrastructure, while platform-specific business logic resides in dedicated projects.","ref":"examples.html#platform-integration-example"},{"type":"extras","title":"Example Patterns - WebsockexAdapter Examples","doc":"","ref":"examples.html#example-patterns"},{"type":"extras","title":"Error Recovery Pattern - WebsockexAdapter Examples","doc":"See [error_handling.ex](../lib/websockex_adapter/examples/docs/error_handling.ex) for a complete implementation:\n\n```elixir\nalias WebsockexAdapter.Examples.Docs.ErrorHandling\n\n# Start a resilient client\n{:ok, client} = ErrorHandling.start_link(\"wss://echo.websocket.org\")\n\n# The client automatically handles:\n# - Connection failures with exponential backoff\n# - Message send failures with retries\n# - WebSocket errors with appropriate recovery\n```","ref":"examples.html#error-recovery-pattern"},{"type":"extras","title":"Subscription Management Pattern - WebsockexAdapter Examples","doc":"See [subscription_management.ex](../lib/websockex_adapter/examples/docs/subscription_management.ex) for implementation:\n\n```elixir\nalias WebsockexAdapter.Examples.Docs.SubscriptionManagement\n\n# Start a client with managed subscriptions\n{:ok, manager} = SubscriptionManagement.start_link(\"wss://test.deribit.com/ws/api/v2\")\n\n# Add subscriptions (automatically restored on reconnection)\n:ok = SubscriptionManagement.add_subscription(manager, \"ticker.BTC-PERPETUAL.raw\")\n:ok = SubscriptionManagement.add_subscription(manager, \"book.ETH-PERPETUAL.100ms\")\n\n# List active subscriptions\nsubscriptions = SubscriptionManagement.list_subscriptions(manager)\n```","ref":"examples.html#subscription-management-pattern"},{"type":"extras","title":"JSON-RPC Pattern - WebsockexAdapter Examples","doc":"See [json_rpc_client.ex](../lib/websockex_adapter/examples/docs/json_rpc_client.ex) for implementation:\n\n```elixir\nalias WebsockexAdapter.Examples.Docs.JsonRpcClient\n\n# Start a JSON-RPC client\n{:ok, client} = JsonRpcClient.start_link(\"wss://api.example.com/jsonrpc\")\n\n# Make RPC calls with automatic correlation\n{:ok, result} = JsonRpcClient.call(client, \"get_account_info\", %{account_id: \"123\"})\n{:ok, balance} = JsonRpcClient.call(client, \"get_balance\", %{currency: \"USD\"})\n```","ref":"examples.html#json-rpc-pattern"},{"type":"extras","title":"Testing Your Implementation - WebsockexAdapter Examples","doc":"All examples come with comprehensive tests. To run them:\n\n```bash\n# Run all example tests\nmix test test/websockex_adapter/examples/\n\n# Run specific example test\nmix test test/websockex_adapter/examples/basic_usage_test.exs\n\n# Run with coverage\nmix test --cover test/websockex_adapter/examples/\n```","ref":"examples.html#testing-your-implementation"},{"type":"extras","title":"Best Practices - WebsockexAdapter Examples","doc":"1. **Use GenServers for Stateful Connections**: See `DeribitGenServerAdapter` for a production-ready pattern\n2. **Handle Errors at the Appropriate Level**: Let the framework handle reconnection, focus on business logic\n3. **Test Against Real APIs**: All examples use real WebSocket endpoints for testing\n4. **Monitor Connection Health**: Use telemetry events and health checks\n5. **Batch Operations**: For high-frequency data, batch updates before processing","ref":"examples.html#best-practices"},{"type":"extras","title":"Extending for Your Platform - WebsockexAdapter Examples","doc":"To create an adapter for your platform:\n\n1. Study the platform adapter template in `examples/platform_adapter_template.ex`\n2. Follow the [adapter building guide](guides/building_adapters.md)\n3. Implement platform-specific:\n   - Authentication flow\n   - Message formatting\n   - Subscription management\n   - Error handling\n4. Add comprehensive tests using real API endpoints\n5. Document platform-specific features","ref":"examples.html#extending-for-your-platform"},{"type":"extras","title":"Additional Resources - WebsockexAdapter Examples","doc":"- [Architecture Overview](Architecture.md)\n- [Building Custom Adapters](guides/building_adapters.md)\n- [Troubleshooting Reconnection](guides/troubleshooting_reconnection.md)\n- [API Documentation](https://hexdocs.pm/websockex_adapter)","ref":"examples.html#additional-resources"},{"type":"extras","title":"Building Exchange Adapters Guide","doc":"# Building Exchange Adapters Guide","ref":"building_adapters.html"},{"type":"extras","title":"Overview - Building Exchange Adapters Guide","doc":"This guide explains how to build exchange adapters for WebsockexAdapter. Exchange adapters provide platform-specific functionality on top of the core WebSocket client, including authentication, subscription management, and state restoration.","ref":"building_adapters.html#overview"},{"type":"extras","title":"Adapter Template - Building Exchange Adapters Guide","doc":"Here's a minimal template for building an exchange adapter:\n\n```elixir\ndefmodule YourExchange.Adapter do\n  use GenServer\n  require Logger\n  \n  alias WebsockexAdapter.Client\n  \n  # Public API\n  \n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: opts[:name])\n  end\n  \n  def connect(adapter) do\n    GenServer.call(adapter, :connect)\n  end\n  \n  def subscribe(adapter, channels) do\n    GenServer.call(adapter, {:subscribe, channels})\n  end\n  \n  def send_order(adapter, order_params) do\n    GenServer.call(adapter, {:send_order, order_params})\n  end\n  \n  # GenServer Callbacks\n  \n  @impl true\n  def init(opts) do\n    state = %{\n      url: opts[:url] || \"wss://your-exchange.com/ws\",\n      client: nil,\n      client_ref: nil,\n      api_key: opts[:api_key],\n      api_secret: opts[:api_secret],\n      subscriptions: MapSet.new(),\n      authenticated: false,\n      reconnecting: false\n    }\n    \n    {:ok, state}\n  end\n  \n  @impl true\n  def handle_call(:connect, _from, state) do\n    case do_connect(state) do\n      {:ok, new_state} ->\n        {:reply, :ok, new_state}\n      {:error, reason} ->\n        {:reply, {:error, reason}, state}\n    end\n  end\n  \n  # Private Functions\n  \n  defp do_connect(state) do\n    # CRITICAL: Always set reconnect_on_error: false\n    connect_opts = [\n      reconnect_on_error: false,  # Adapter handles reconnection\n      heartbeat_config: %{\n        type: :custom,  # or :deribit, :standard\n        interval: 30_000\n      }\n    ]\n    \n    case Client.connect(state.url, connect_opts) do\n      {:ok, client} ->\n        # Monitor the client process\n        ref = Process.monitor(client.server_pid)\n        \n        new_state = %{state | \n          client: client, \n          client_ref: ref,\n          reconnecting: false\n        }\n        \n        # Authenticate if credentials provided\n        case authenticate(new_state) do\n          {:ok, auth_state} ->\n            # Restore subscriptions if any\n            restore_subscriptions(auth_state)\n          error ->\n            error\n        end\n        \n      {:error, reason} ->\n        {:error, reason}\n    end\n  end\n  \n  defp authenticate(state) when is_nil(state.api_key) do\n    {:ok, state}  # No authentication needed\n  end\n  \n  defp authenticate(state) do\n    # Exchange-specific authentication\n    auth_msg = build_auth_message(state.api_key, state.api_secret)\n    \n    case Client.send_message(state.client, auth_msg) do\n      :ok ->\n        # Wait for auth response (simplified)\n        {:ok, %{state | authenticated: true}}\n      error ->\n        error\n    end\n  end\n  \n  defp restore_subscriptions(%{subscriptions: subs} = state) when subs != %MapSet{} do\n    Enum.each(subs, fn channel ->\n      sub_msg = build_subscription_message(channel)\n      Client.send_message(state.client, sub_msg)\n    end)\n    \n    {:ok, state}\n  end\n  \n  defp restore_subscriptions(state), do: {:ok, state}\n  \n  # Monitor handling - CRITICAL for reconnection\n  \n  @impl true\n  def handle_info({:DOWN, ref, :process, _pid, reason}, %{client_ref: ref} = state) do\n    Logger.warn(\"Client process down: #{inspect(reason)}, initiating reconnection\")\n    \n    new_state = %{state | \n      client: nil, \n      client_ref: nil,\n      reconnecting: true,\n      authenticated: false\n    }\n    \n    # Attempt immediate reconnection\n    case do_connect(new_state) do\n      {:ok, connected_state} ->\n        Logger.info(\"Successfully reconnected\")\n        {:noreply, connected_state}\n      {:error, reason} ->\n        Logger.error(\"Reconnection failed: #{inspect(reason)}\")\n        # Schedule retry\n        Process.send_after(self(), :retry_connect, 5_000)\n        {:noreply, new_state}\n    end\n  end\n  \n  @impl true\n  def handle_info(:retry_connect, state) do\n    case do_connect(state) do\n      {:ok, connected_state} ->\n        {:noreply, connected_state}\n      {:error, _} ->\n        # Exponential backoff could be implemented here\n        Process.send_after(self(), :retry_connect, 10_000)\n        {:noreply, state}\n    end\n  end\n  \n  # Exchange-specific message builders\n  \n  defp build_auth_message(api_key, api_secret) do\n    # Exchange-specific auth format\n    %{\n      \"method\" => \"auth\",\n      \"params\" => %{\n        \"api_key\" => api_key,\n        \"signature\" => generate_signature(api_secret)\n      }\n    }\n  end\n  \n  defp build_subscription_message(channel) do\n    # Exchange-specific subscription format\n    %{\n      \"method\" => \"subscribe\",\n      \"params\" => %{\n        \"channel\" => channel\n      }\n    }\n  end\nend\n```","ref":"building_adapters.html#adapter-template"},{"type":"extras","title":"Critical Implementation Rules - Building Exchange Adapters Guide","doc":"","ref":"building_adapters.html#critical-implementation-rules"},{"type":"extras","title":"1. Always Set reconnect_on_error: false - Building Exchange Adapters Guide","doc":"This is the most critical rule. Your adapter MUST disable the Client's internal reconnection:\n\n```elixir\nconnect_opts = [\n  reconnect_on_error: false,  # REQUIRED for adapters\n  # ... other options\n]\n```\n\n**Why?** This prevents duplicate reconnection attempts. The adapter handles all reconnection logic.","ref":"building_adapters.html#1-always-set-reconnect_on_error-false"},{"type":"extras","title":"2. Monitor the Client Process - Building Exchange Adapters Guide","doc":"Always monitor the Client process to detect failures:\n\n```elixir\n{:ok, client} = Client.connect(url, opts)\nref = Process.monitor(client.server_pid)\n```","ref":"building_adapters.html#2-monitor-the-client-process"},{"type":"extras","title":"3. Handle Process DOWN Messages - Building Exchange Adapters Guide","doc":"Implement proper handling for client process termination:\n\n```elixir\ndef handle_info({:DOWN, ref, :process, _pid, reason}, %{client_ref: ref} = state) do\n  # Client died - initiate reconnection\n  # This is your ONLY reconnection trigger\nend\n```","ref":"building_adapters.html#3-handle-process-down-messages"},{"type":"extras","title":"State Restoration Pattern - Building Exchange Adapters Guide","doc":"After reconnection, restore your application state in order:\n\n1. **Re-establish connection** (creates new Gun process)\n2. **Authenticate** (if required by exchange)\n3. **Restore subscriptions** (market data, account updates)\n4. **Resume operations** (re-enable trading, etc.)\n\n```elixir\ndefp restore_connection_state(state) do\n  with {:ok, connected_state} <- establish_connection(state),\n       {:ok, auth_state} <- authenticate(connected_state),\n       {:ok, sub_state} <- restore_subscriptions(auth_state),\n       {:ok, final_state} <- resume_operations(sub_state) do\n    {:ok, final_state}\n  end\nend\n```","ref":"building_adapters.html#state-restoration-pattern"},{"type":"extras","title":"Example: Deribit Adapter - Building Exchange Adapters Guide","doc":"Study the production-ready Deribit adapter for a complete implementation:\n\n```elixir\n# From lib/websockex_adapter/examples/deribit_genserver_adapter.ex\n\ndefp do_connect(state) do\n  # Parse URL for testnet/mainnet\n  url = state.url || @default_url\n  \n  # Configure connection\n  connect_opts = [\n    heartbeat_config: %{\n      type: :deribit,\n      interval: heartbeat_interval\n    },\n    reconnect_on_error: false  # Critical!\n  ]\n  \n  # Connect and monitor\n  case Client.connect(url, connect_opts) do\n    {:ok, client} ->\n      ref = Process.monitor(client.server_pid)\n      \n      new_state = %{state | \n        client: client, \n        monitor_ref: ref,\n        connected: true,\n        connecting: false\n      }\n      \n      # Authenticate if we have credentials\n      if state.client_id && state.client_secret do\n        case authenticate(new_state) do\n          {:ok, auth_state} ->\n            restore_subscriptions(auth_state)\n          {:error, reason} ->\n            {:error, reason}\n        end\n      else\n        {:ok, new_state}\n      end\n  end\nend\n```","ref":"building_adapters.html#example-deribit-adapter"},{"type":"extras","title":"Common Patterns - Building Exchange Adapters Guide","doc":"","ref":"building_adapters.html#common-patterns"},{"type":"extras","title":"Authentication Flow - Building Exchange Adapters Guide","doc":"```elixir\ndefp authenticate(state) do\n  auth_params = %{\n    \"jsonrpc\" => \"2.0\",\n    \"method\" => \"public/auth\",\n    \"params\" => %{\n      \"grant_type\" => \"client_credentials\",\n      \"client_id\" => state.client_id,\n      \"client_secret\" => state.client_secret\n    },\n    \"id\" => generate_id()\n  }\n  \n  case Client.send_message(state.client, auth_params) do\n    :ok ->\n      # Mark as authenticating, wait for response\n      {:ok, %{state | authenticating: true}}\n    error ->\n      error\n  end\nend\n```","ref":"building_adapters.html#authentication-flow"},{"type":"extras","title":"Subscription Management - Building Exchange Adapters Guide","doc":"```elixir\ndefp track_subscription(state, channel) do\n  new_subs = MapSet.put(state.subscriptions, channel)\n  %{state | subscriptions: new_subs}\nend\n\ndefp restore_subscriptions(state) do\n  Enum.each(state.subscriptions, fn channel ->\n    subscribe_message = build_subscribe_message(channel)\n    Client.send_message(state.client, subscribe_message)\n  end)\n  \n  {:ok, state}\nend\n```","ref":"building_adapters.html#subscription-management"},{"type":"extras","title":"Cancel-on-Disconnect - Building Exchange Adapters Guide","doc":"```elixir\ndefp handle_connection_loss(state) do\n  if state.cancel_on_disconnect do\n    # Cancel all open orders\n    cancel_all_orders(state)\n  end\n  \n  # Proceed with reconnection\n  initiate_reconnection(state)\nend\n```","ref":"building_adapters.html#cancel-on-disconnect"},{"type":"extras","title":"Testing Your Adapter - Building Exchange Adapters Guide","doc":"","ref":"building_adapters.html#testing-your-adapter"},{"type":"extras","title":"Unit Tests - Building Exchange Adapters Guide","doc":"```elixir\ndefmodule YourExchange.AdapterTest do\n  use ExUnit.Case\n  \n  alias YourExchange.Adapter\n  \n  describe \"reconnection handling\" do\n    test \"reconnects on client process death\" do\n      {:ok, adapter} = Adapter.start_link(url: \"wss://test.exchange.com\")\n      \n      # Connect\n      assert :ok = Adapter.connect(adapter)\n      \n      # Get client process\n      state = :sys.get_state(adapter)\n      client_pid = state.client.server_pid\n      \n      # Kill client process\n      Process.exit(client_pid, :kill)\n      \n      # Adapter should reconnect\n      :timer.sleep(100)\n      \n      new_state = :sys.get_state(adapter)\n      assert new_state.client != nil\n      assert new_state.client.server_pid != client_pid\n    end\n  end\nend\n```","ref":"building_adapters.html#unit-tests"},{"type":"extras","title":"Integration Tests - Building Exchange Adapters Guide","doc":"```elixir\n@tag :integration\ntest \"maintains subscriptions across reconnection\" do\n  {:ok, adapter} = Adapter.start_link(\n    url: \"wss://test.exchange.com\",\n    api_key: \"test_key\",\n    api_secret: \"test_secret\"\n  )\n  \n  # Connect and subscribe\n  :ok = Adapter.connect(adapter)\n  :ok = Adapter.subscribe(adapter, [\"trades.BTC-USD\"])\n  \n  # Force disconnection\n  state = :sys.get_state(adapter)\n  Process.exit(state.client.server_pid, :kill)\n  \n  # Wait for reconnection\n  :timer.sleep(1000)\n  \n  # Verify subscription restored\n  assert subscription_active?(adapter, \"trades.BTC-USD\")\nend\n```","ref":"building_adapters.html#integration-tests"},{"type":"extras","title":"Troubleshooting - Building Exchange Adapters Guide","doc":"","ref":"building_adapters.html#troubleshooting"},{"type":"extras","title":"Common Issues - Building Exchange Adapters Guide","doc":"1. **Duplicate Reconnection Attempts**\n   - Symptom: Multiple connection attempts, resource exhaustion\n   - Solution: Ensure `reconnect_on_error: false` is set\n\n2. **Lost Subscriptions**\n   - Symptom: No data after reconnection\n   - Solution: Track subscriptions in adapter state, restore after auth\n\n3. **Authentication Failures**\n   - Symptom: Can't restore authenticated state\n   - Solution: Store credentials securely, handle auth errors gracefully\n\n4. **Memory Leaks**\n   - Symptom: Growing process count\n   - Solution: Ensure old monitors are cleaned up, Gun processes terminate","ref":"building_adapters.html#common-issues"},{"type":"extras","title":"Best Practices - Building Exchange Adapters Guide","doc":"1. **Use Supervisors**: Always run adapters under a Supervisor\n2. **Log Transitions**: Log all state changes for debugging\n3. **Implement Backoff**: Use exponential backoff for reconnection attempts\n4. **Monitor Health**: Add telemetry for connection health metrics\n5. **Test Failures**: Test with real network failures and process crashes\n6. **Handle Partial State**: Be prepared for partial message delivery","ref":"building_adapters.html#best-practices"},{"type":"extras","title":"Summary - Building Exchange Adapters Guide","doc":"Building a robust exchange adapter requires:\n- Disabling Client reconnection (`reconnect_on_error: false`)\n- Monitoring Client process for failures\n- Implementing complete state restoration\n- Testing reconnection scenarios thoroughly\n\nFollow the patterns in this guide and study the Deribit adapter example for a production-ready implementation.","ref":"building_adapters.html#summary"},{"type":"extras","title":"Troubleshooting Reconnection Guide","doc":"# Troubleshooting Reconnection Guide","ref":"troubleshooting_reconnection.html"},{"type":"extras","title":"Overview - Troubleshooting Reconnection Guide","doc":"This guide helps diagnose and resolve common reconnection issues in WebsockexAdapter. It covers debugging techniques, performance considerations, and monitoring strategies for production deployments.","ref":"troubleshooting_reconnection.html#overview"},{"type":"extras","title":"Common Reconnection Issues - Troubleshooting Reconnection Guide","doc":"","ref":"troubleshooting_reconnection.html#common-reconnection-issues"},{"type":"extras","title":"1. Duplicate Reconnection Attempts - Troubleshooting Reconnection Guide","doc":"**Symptoms:**\n- Multiple connection attempts in logs\n- Resource exhaustion (too many processes)\n- Rapid connection/disconnection cycles\n\n**Root Cause:**\nBoth Client and Adapter attempting reconnection independently.\n\n**Solution:**\n```elixir\n# In your adapter's do_connect function\nconnect_opts = [\n  reconnect_on_error: false,  # MUST be false for adapters\n  heartbeat_config: %{...}\n]\n```\n\n**Verification:**\n```elixir\n# Check client configuration\n{:ok, state} = Client.get_state(client)\nIO.inspect(state.config.reconnect_on_error)  # Should be false\n```","ref":"troubleshooting_reconnection.html#1-duplicate-reconnection-attempts"},{"type":"extras","title":"2. Lost Subscriptions After Reconnection - Troubleshooting Reconnection Guide","doc":"**Symptoms:**\n- No market data after reconnection\n- Missing account updates\n- Silent connection (no data flow)\n\n**Root Cause:**\nAdapter not tracking or restoring subscriptions.\n\n**Solution:**\n```elixir\ndefmodule YourAdapter do\n  # Track subscriptions in state\n  defstruct [..., subscriptions: MapSet.new()]\n  \n  def handle_call({:subscribe, channels}, _from, state) do\n    # Send subscription request\n    Client.send_message(state.client, build_sub_msg(channels))\n    \n    # Track in state\n    new_subs = MapSet.union(state.subscriptions, MapSet.new(channels))\n    {:reply, :ok, %{state | subscriptions: new_subs}}\n  end\n  \n  # Restore after reconnection\n  defp restore_subscriptions(state) do\n    Enum.each(state.subscriptions, fn channel ->\n      Client.send_message(state.client, build_sub_msg([channel]))\n    end)\n  end\nend\n```","ref":"troubleshooting_reconnection.html#2-lost-subscriptions-after-reconnection"},{"type":"extras","title":"3. Authentication Failures on Reconnection - Troubleshooting Reconnection Guide","doc":"**Symptoms:**\n- Reconnection succeeds but authenticated endpoints fail\n- \"Unauthorized\" errors after reconnection\n- Orders rejected after reconnection\n\n**Root Cause:**\nAuthentication state not restored after creating new connection.\n\n**Solution:**\n```elixir\n# Store credentials securely\ndefmodule YourAdapter do\n  defstruct [\n    ...,\n    client_id: nil,\n    client_secret: nil,\n    auth_token: nil,\n    auth_expiry: nil\n  ]\n  \n  defp do_connect(state) do\n    with {:ok, client} <- Client.connect(url, opts),\n         {:ok, auth_state} <- authenticate(client, state),\n         {:ok, final_state} <- restore_subscriptions(auth_state) do\n      {:ok, final_state}\n    end\n  end\n  \n  defp authenticate(client, state) do\n    # Re-authenticate with stored credentials\n    auth_msg = build_auth_message(state.client_id, state.client_secret)\n    Client.send_message(client, auth_msg)\n    # ... handle response\n  end\nend\n```","ref":"troubleshooting_reconnection.html#3-authentication-failures-on-reconnection"},{"type":"extras","title":"4. Memory Leaks from Failed Connections - Troubleshooting Reconnection Guide","doc":"**Symptoms:**\n- Growing process count\n- Increasing memory usage\n- Eventually: system out of memory\n\n**Root Cause:**\nGun processes not being properly cleaned up.\n\n**Solution:**\n```elixir\n# Always demonitor before creating new connection\ndefp cleanup_old_connection(state) do\n  if state.monitor_ref do\n    Process.demonitor(state.monitor_ref, [:flush])\n  end\n  \n  %{state | client: nil, monitor_ref: nil}\nend\n\ndefp do_connect(state) do\n  # Clean up first\n  clean_state = cleanup_old_connection(state)\n  \n  # Then connect\n  case Client.connect(url, opts) do\n    {:ok, client} ->\n      ref = Process.monitor(client.server_pid)\n      %{clean_state | client: client, monitor_ref: ref}\n  end\nend\n```","ref":"troubleshooting_reconnection.html#4-memory-leaks-from-failed-connections"},{"type":"extras","title":"5. Reconnection Storms - Troubleshooting Reconnection Guide","doc":"**Symptoms:**\n- Hundreds of reconnection attempts\n- Server rejecting connections\n- Rate limiting or IP bans\n\n**Root Cause:**\nNo backoff strategy or circuit breaker.\n\n**Solution:**\n```elixir\ndefmodule YourAdapter do\n  defstruct [\n    ...,\n    reconnect_attempts: 0,\n    last_reconnect: nil,\n    max_attempts: 10\n  ]\n  \n  def handle_info({:DOWN, ref, :process, _pid, _reason}, state) do\n    if state.reconnect_attempts >= state.max_attempts do\n      Logger.error(\"Max reconnection attempts reached, giving up\")\n      {:stop, :max_reconnections_exceeded, state}\n    else\n      # Exponential backoff\n      delay = calculate_backoff(state.reconnect_attempts)\n      Process.send_after(self(), :reconnect, delay)\n      \n      {:noreply, %{state | \n        reconnect_attempts: state.reconnect_attempts + 1,\n        last_reconnect: DateTime.utc_now()\n      }}\n    end\n  end\n  \n  defp calculate_backoff(attempts) do\n    # 1s, 2s, 4s, 8s, 16s, 32s, 60s max\n    min(1000 * :math.pow(2, attempts), 60_000)\n  end\nend\n```","ref":"troubleshooting_reconnection.html#5-reconnection-storms"},{"type":"extras","title":"Debugging Connection Failures - Troubleshooting Reconnection Guide","doc":"","ref":"troubleshooting_reconnection.html#debugging-connection-failures"},{"type":"extras","title":"Enable Detailed Logging - Troubleshooting Reconnection Guide","doc":"```elixir\n# In your config/config.exs\nconfig :logger, :console,\n  level: :debug,\n  format: \"$time $metadata[$level] $message\\n\",\n  metadata: [:module, :function, :line]\n\n# Add logging to your adapter\ndefmodule YourAdapter do\n  require Logger\n  \n  defp do_connect(state) do\n    Logger.debug(\"Attempting connection to #{state.url}\")\n    \n    case Client.connect(state.url, opts) do\n      {:ok, client} ->\n        Logger.info(\"Successfully connected\")\n        {:ok, client}\n      {:error, reason} ->\n        Logger.error(\"Connection failed: #{inspect(reason)}\")\n        {:error, reason}\n    end\n  end\nend\n```","ref":"troubleshooting_reconnection.html#enable-detailed-logging"},{"type":"extras","title":"Use Telemetry Events - Troubleshooting Reconnection Guide","doc":"```elixir\n# Emit telemetry events\n:telemetry.execute(\n  [:your_app, :websocket, :reconnection, :attempt],\n  %{count: state.reconnect_attempts},\n  %{url: state.url, reason: reason}\n)\n\n# Monitor in another process\n:telemetry.attach(\n  \"websocket-reconnection-monitor\",\n  [:your_app, :websocket, :reconnection, :attempt],\n  fn _event, measurements, metadata, _config ->\n    IO.puts(\"Reconnection attempt #{measurements.count} for #{metadata.url}\")\n  end,\n  nil\n)\n```","ref":"troubleshooting_reconnection.html#use-telemetry-events"},{"type":"extras","title":"Inspect Process State - Troubleshooting Reconnection Guide","doc":"```elixir\n# During runtime debugging\nstate = :sys.get_state(your_adapter_pid)\nIO.inspect(state, label: \"Adapter state\")\n\n# Check if client is alive\nif state.client do\n  client_state = Client.get_state(state.client)\n  IO.inspect(client_state, label: \"Client state\")\nend\n\n# List all Gun connections\n:gun.info()\n|> Enum.each(fn {pid, info} ->\n  IO.inspect({pid, info}, label: \"Gun connection\")\nend)\n```","ref":"troubleshooting_reconnection.html#inspect-process-state"},{"type":"extras","title":"Performance Considerations - Troubleshooting Reconnection Guide","doc":"","ref":"troubleshooting_reconnection.html#performance-considerations"},{"type":"extras","title":"1. Connection Pooling - Troubleshooting Reconnection Guide","doc":"For high-frequency reconnection scenarios:\n\n```elixir\ndefmodule ConnectionPool do\n  use GenServer\n  \n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n  \n  def get_connection() do\n    GenServer.call(__MODULE__, :get_connection)\n  end\n  \n  def init(opts) do\n    # Pre-create connections\n    connections = for _ <- 1..opts[:size] do\n      {:ok, client} = Client.connect(opts[:url], reconnect_on_error: false)\n      client\n    end\n    \n    {:ok, %{connections: connections, available: connections}}\n  end\nend\n```","ref":"troubleshooting_reconnection.html#1-connection-pooling"},{"type":"extras","title":"2. Circuit Breaker Pattern - Troubleshooting Reconnection Guide","doc":"Prevent cascading failures:\n\n```elixir\ndefmodule CircuitBreaker do\n  defstruct [\n    state: :closed,  # :closed, :open, :half_open\n    failure_count: 0,\n    failure_threshold: 5,\n    timeout: 60_000,\n    last_failure: nil\n  ]\n  \n  def call(breaker, fun) do\n    case breaker.state do\n      :open ->\n        if time_to_retry?(breaker) do\n          try_half_open(breaker, fun)\n        else\n          {:error, :circuit_open}\n        end\n      \n      :closed ->\n        execute_with_breaker(breaker, fun)\n      \n      :half_open ->\n        execute_with_breaker(breaker, fun)\n    end\n  end\nend\n```","ref":"troubleshooting_reconnection.html#2-circuit-breaker-pattern"},{"type":"extras","title":"3. Resource Monitoring - Troubleshooting Reconnection Guide","doc":"Monitor system resources:\n\n```elixir\ndefmodule ResourceMonitor do\n  def check_resources() do\n    %{\n      process_count: :erlang.system_info(:process_count),\n      memory: :erlang.memory(:total),\n      gun_connections: length(:gun.info()),\n      ets_tables: length(:ets.all())\n    }\n  end\n  \n  def alert_if_high() do\n    resources = check_resources()\n    \n    if resources.process_count > 10_000 do\n      Logger.error(\"High process count: #{resources.process_count}\")\n    end\n    \n    if resources.gun_connections > 100 do\n      Logger.error(\"High Gun connection count: #{resources.gun_connections}\")\n    end\n  end\nend\n```","ref":"troubleshooting_reconnection.html#3-resource-monitoring"},{"type":"extras","title":"Monitoring Recommendations - Troubleshooting Reconnection Guide","doc":"","ref":"troubleshooting_reconnection.html#monitoring-recommendations"},{"type":"extras","title":"1. StatsD/Prometheus Metrics - Troubleshooting Reconnection Guide","doc":"```elixir\ndefmodule Metrics do\n  def record_reconnection(reason) do\n    :telemetry.execute(\n      [:websocket, :reconnection],\n      %{count: 1},\n      %{reason: reason}\n    )\n  end\n  \n  def record_connection_duration(start_time) do\n    duration = System.monotonic_time() - start_time\n    \n    :telemetry.execute(\n      [:websocket, :connection, :duration],\n      %{duration: duration},\n      %{}\n    )\n  end\nend\n```","ref":"troubleshooting_reconnection.html#1-statsd-prometheus-metrics"},{"type":"extras","title":"2. Health Checks - Troubleshooting Reconnection Guide","doc":"```elixir\ndefmodule HealthCheck do\n  def check_websocket_health(adapter) do\n    case GenServer.call(adapter, :get_state, 5000) do\n      {:ok, state} when state.connected ->\n        {:ok, \"WebSocket connected\"}\n      \n      {:ok, state} ->\n        {:error, \"WebSocket disconnected\", state}\n      \n      {:error, :timeout} ->\n        {:error, \"Adapter not responding\"}\n    end\n  end\nend\n```","ref":"troubleshooting_reconnection.html#2-health-checks"},{"type":"extras","title":"3. Alerting Rules - Troubleshooting Reconnection Guide","doc":"Set up alerts for:\n- Connection failure rate > 10% over 5 minutes\n- Average reconnection time > 30 seconds\n- Process count growth > 1000 per hour\n- Memory usage growth > 100MB per hour","ref":"troubleshooting_reconnection.html#3-alerting-rules"},{"type":"extras","title":"Testing Reconnection Scenarios - Troubleshooting Reconnection Guide","doc":"","ref":"troubleshooting_reconnection.html#testing-reconnection-scenarios"},{"type":"extras","title":"Simulate Network Failures - Troubleshooting Reconnection Guide","doc":"```elixir\ndefmodule NetworkSimulator do\n  def drop_connection(client) do\n    # Get Gun pid from client\n    state = Client.get_state(client)\n    Process.exit(state.gun_pid, :kill)\n  end\n  \n  def block_traffic(duration) do\n    # Use iptables or similar to block traffic\n    System.cmd(\"sudo\", [\"iptables\", \"-A\", \"OUTPUT\", \"-p\", \"tcp\", \"--dport\", \"443\", \"-j\", \"DROP\"])\n    :timer.sleep(duration)\n    System.cmd(\"sudo\", [\"iptables\", \"-D\", \"OUTPUT\", \"-p\", \"tcp\", \"--dport\", \"443\", \"-j\", \"DROP\"])\n  end\nend\n```","ref":"troubleshooting_reconnection.html#simulate-network-failures"},{"type":"extras","title":"Load Testing - Troubleshooting Reconnection Guide","doc":"```elixir\ndefmodule LoadTest do\n  def stress_reconnection(adapter, iterations) do\n    for i <- 1..iterations do\n      Task.start(fn ->\n        # Force disconnection\n        state = :sys.get_state(adapter)\n        Process.exit(state.client.server_pid, :kill)\n        \n        # Wait for reconnection\n        :timer.sleep(1000)\n        \n        # Verify reconnected\n        new_state = :sys.get_state(adapter)\n        assert new_state.connected\n      end)\n      \n      :timer.sleep(100)  # Stagger disconnections\n    end\n  end\nend\n```","ref":"troubleshooting_reconnection.html#load-testing"},{"type":"extras","title":"Quick Diagnosis Checklist - Troubleshooting Reconnection Guide","doc":"When experiencing reconnection issues:\n\n1. **Check Configuration**\n   ```elixir\n   # Is reconnect_on_error set correctly?\n   IO.inspect(state.config.reconnect_on_error)\n   ```\n\n2. **Verify Process Monitoring**\n   ```elixir\n   # Is the monitor ref valid?\n   IO.inspect(Process.info(self(), :monitors))\n   ```\n\n3. **Check Gun Processes**\n   ```elixir\n   # How many Gun connections exist?\n   IO.inspect(length(:gun.info()))\n   ```\n\n4. **Review Logs**\n   ```bash\n   grep -i \"reconnect\\|connection\\|gun\" app.log | tail -100\n   ```\n\n5. **Monitor Resources**\n   ```elixir\n   # Check system resources\n   IO.inspect(:erlang.memory())\n   IO.inspect(:erlang.system_info(:process_count))\n   ```","ref":"troubleshooting_reconnection.html#quick-diagnosis-checklist"},{"type":"extras","title":"Summary - Troubleshooting Reconnection Guide","doc":"Successful reconnection troubleshooting requires:\n1. Understanding the dual-layer architecture\n2. Proper configuration (`reconnect_on_error: false` for adapters)\n3. Comprehensive logging and monitoring\n4. Testing failure scenarios\n5. Resource management and cleanup\n\nFollow this guide's patterns to build robust, production-ready WebSocket connections for financial trading systems.","ref":"troubleshooting_reconnection.html#summary"},{"type":"extras","title":"WebsockexAdapter Architecture","doc":"# WebsockexAdapter Architecture","ref":"architecture.html"},{"type":"extras","title":"Overview - WebsockexAdapter Architecture","doc":"WebsockexAdapter is a production-grade WebSocket client library built on top of the Gun HTTP/2 client. It provides a simple, reliable interface for WebSocket communications with a focus on financial trading systems.","ref":"architecture.html#overview"},{"type":"extras","title":"Core Design Principles - WebsockexAdapter Architecture","doc":"1. **Simplicity First** - Maximum 5 functions per module, 15 lines per function\n2. **Real-World Testing** - No mocks, only real API testing\n3. **Financial-Grade Reliability** - Built for high-frequency trading systems\n4. **Minimal Abstraction** - Direct Gun API usage, no unnecessary wrappers","ref":"architecture.html#core-design-principles"},{"type":"extras","title":"Architecture Layers - WebsockexAdapter Architecture","doc":"","ref":"architecture.html#architecture-layers"},{"type":"extras","title":"1. Transport Layer (Gun) - WebsockexAdapter Architecture","doc":"- Direct integration with Gun for WebSocket connections\n- HTTP/2 support for modern protocols\n- Connection pooling and multiplexing","ref":"architecture.html#1-transport-layer-gun"},{"type":"extras","title":"2. Core Modules - WebsockexAdapter Architecture","doc":"#### Client (`client.ex`)\nThe main interface for WebSocket operations:\n- `connect/2` - Establish WebSocket connection\n- `send_message/2` - Send messages to server\n- `close/1` - Close connection gracefully\n- `subscribe/2` - Subscribe to data channels\n- `get_state/1` - Retrieve connection state\n\n#### Frame (`frame.ex`)\nWebSocket frame handling:\n- Binary frame encoding/decoding\n- Text frame support\n- Control frame processing\n- Fragmented message assembly\n\n#### Reconnection (`reconnection.ex`)\nAutomatic reconnection with exponential backoff:\n- Configurable retry attempts\n- Exponential backoff calculation\n- State preservation across reconnections\n- Connection failure categorization\n\n#### Message Handler (`message_handler.ex`)\nMessage routing and processing:\n- Incoming message parsing\n- Message type detection\n- Callback routing\n- Error message handling\n\n#### Error Handler (`error_handler.ex`)\nComprehensive error management:\n- Error categorization (connection, protocol, auth, application)\n- Recovery strategy selection\n- Error context preservation\n- Logging and telemetry","ref":"architecture.html#2-core-modules"},{"type":"extras","title":"3. Protocol Support - WebsockexAdapter Architecture","doc":"#### JSON-RPC (`json_rpc.ex`)\nFull JSON-RPC 2.0 implementation:\n- Request/response correlation\n- Batch request support\n- Notification handling\n- Error response parsing","ref":"architecture.html#3-protocol-support"},{"type":"extras","title":"4. Infrastructure Modules - WebsockexAdapter Architecture","doc":"#### Connection Registry (`connection_registry.ex`)\nETS-based connection tracking:\n- Fast connection lookups\n- Multi-connection support\n- Connection metadata storage\n- Cleanup on termination\n\n#### Rate Limiter (`rate_limiter.ex`)\nToken bucket rate limiting:\n- Configurable rate limits\n- Burst capacity support\n- Per-connection limiting\n- Exchange-specific configurations","ref":"architecture.html#4-infrastructure-modules"},{"type":"extras","title":"5. Platform Adapters - WebsockexAdapter Architecture","doc":"#### Deribit Adapter (`examples/deribit_adapter.ex`)\nReference implementation for exchange integration:\n- Authentication flow\n- Heartbeat management\n- Subscription handling\n- Order management\n- Market data processing","ref":"architecture.html#5-platform-adapters"},{"type":"extras","title":"Data Flow - WebsockexAdapter Architecture","doc":"```\nUser Code\n    |\n    v\nClient API (5 functions)\n    |\n    v\nMessage Handler <---> JSON-RPC\n    |                    |\n    v                    v\nFrame Encoder      Rate Limiter\n    |                    |\n    v                    v\nGun Transport <---> WebSocket Server\n    |\n    v\nError Handler --> Reconnection\n```","ref":"architecture.html#data-flow"},{"type":"extras","title":"State Management - WebsockexAdapter Architecture","doc":"","ref":"architecture.html#state-management"},{"type":"extras","title":"Connection State - WebsockexAdapter Architecture","doc":"- Managed by Client GenServer\n- Includes: connection status, subscriptions, pending requests\n- Preserved across reconnections","ref":"architecture.html#connection-state"},{"type":"extras","title":"Registry State - WebsockexAdapter Architecture","doc":"- ETS table for O(1) lookups\n- Stores: PID to connection mappings\n- Automatic cleanup on process termination","ref":"architecture.html#registry-state"},{"type":"extras","title":"Rate Limiter State - WebsockexAdapter Architecture","doc":"- Token bucket per connection\n- Configurable refill rates\n- Burst capacity tracking","ref":"architecture.html#rate-limiter-state"},{"type":"extras","title":"Error Handling Strategy - WebsockexAdapter Architecture","doc":"1. **Connection Errors**: Trigger automatic reconnection\n2. **Protocol Errors**: Log and notify user callback\n3. **Authentication Errors**: Halt and require user intervention\n4. **Application Errors**: Pass through to user code","ref":"architecture.html#error-handling-strategy"},{"type":"extras","title":"Supervision Strategy - WebsockexAdapter Architecture","doc":"","ref":"architecture.html#supervision-strategy"},{"type":"extras","title":"Client Supervisor - WebsockexAdapter Architecture","doc":"- Simple one-for-one strategy\n- Restart clients on failure\n- Configurable restart intensity","ref":"architecture.html#client-supervisor"},{"type":"extras","title":"Adapter Supervision - WebsockexAdapter Architecture","doc":"- Platform adapters handle their own supervision\n- Separation of concerns between transport and business logic\n- Clean restart semantics","ref":"architecture.html#adapter-supervision"},{"type":"extras","title":"Performance Considerations - WebsockexAdapter Architecture","doc":"1. **ETS for Fast Lookups**: Connection registry uses ETS\n2. **Direct Gun API**: No abstraction overhead\n3. **Efficient Frame Processing**: Minimal allocations\n4. **Telemetry Integration**: Observable performance metrics","ref":"architecture.html#performance-considerations"},{"type":"extras","title":"Extension Points - WebsockexAdapter Architecture","doc":"","ref":"architecture.html#extension-points"},{"type":"extras","title":"Custom Adapters - WebsockexAdapter Architecture","doc":"1. Implement authentication for your platform\n2. Handle platform-specific message formats\n3. Add custom subscription logic\n4. Integrate with platform features","ref":"architecture.html#custom-adapters"},{"type":"extras","title":"Custom Protocols - WebsockexAdapter Architecture","doc":"1. Extend message handler for new formats\n2. Add protocol-specific frame handling\n3. Implement custom correlation strategies","ref":"architecture.html#custom-protocols"},{"type":"extras","title":"Testing Architecture - WebsockexAdapter Architecture","doc":"","ref":"architecture.html#testing-architecture"},{"type":"extras","title":"Unit Tests - WebsockexAdapter Architecture","doc":"- Test individual modules in isolation\n- Use local mock servers (not mocks!)\n- Verify edge cases and error conditions","ref":"architecture.html#unit-tests"},{"type":"extras","title":"Integration Tests - WebsockexAdapter Architecture","doc":"- Test against real APIs (test.deribit.com)\n- Verify end-to-end functionality\n- Test reconnection scenarios\n- Measure real-world performance","ref":"architecture.html#integration-tests"},{"type":"extras","title":"Stability Tests - WebsockexAdapter Architecture","doc":"- Long-running connection tests\n- High-frequency message testing\n- Network interruption simulation\n- Memory leak detection","ref":"architecture.html#stability-tests"},{"type":"extras","title":"Security Considerations - WebsockexAdapter Architecture","doc":"1. **TLS by Default**: All connections use TLS\n2. **Credential Management**: Environment variables for secrets\n3. **No Credential Logging**: Sensitive data never logged\n4. **Secure Frame Masking**: Client-side frame masking","ref":"architecture.html#security-considerations"},{"type":"extras","title":"Monitoring and Observability - WebsockexAdapter Architecture","doc":"","ref":"architecture.html#monitoring-and-observability"},{"type":"extras","title":"Telemetry Events - WebsockexAdapter Architecture","doc":"- Connection lifecycle events\n- Message send/receive metrics\n- Error occurrence tracking\n- Performance measurements","ref":"architecture.html#telemetry-events"},{"type":"extras","title":"Health Checks - WebsockexAdapter Architecture","doc":"- Connection state monitoring\n- Heartbeat status\n- Rate limit utilization\n- Queue depth tracking","ref":"architecture.html#health-checks"},{"type":"extras","title":"Reconnection Architecture Guide","doc":"# Reconnection Architecture Guide","ref":"reconnection.html"},{"type":"extras","title":"Overview - Reconnection Architecture Guide","doc":"WebsockexAdapter implements a dual-layer reconnection architecture that provides both network-level and process-level reliability. This design leverages Gun's flexible process ownership model to ensure robust connection management for financial trading systems.","ref":"reconnection.html#overview"},{"type":"extras","title":"Dual-Layer Design Rationale - Reconnection Architecture Guide","doc":"","ref":"reconnection.html#dual-layer-design-rationale"},{"type":"extras","title":"Layer 1: Client-Level Reconnection (Network Recovery) - Reconnection Architecture Guide","doc":"The `WebsockexAdapter.Client` GenServer provides built-in reconnection for network-level failures:\n- Handles temporary network outages\n- Manages WebSocket protocol errors\n- Preserves connection state during recovery\n- Configurable via `reconnect_on_error` option (default: true)","ref":"reconnection.html#layer-1-client-level-reconnection-network-recovery"},{"type":"extras","title":"Layer 2: Adapter-Level Supervision (Process Recovery) - Reconnection Architecture Guide","doc":"Platform adapters like `DeribitGenServerAdapter` provide process-level supervision:\n- Monitors Client process health\n- Handles complete process crashes\n- Restores authentication state\n- Re-establishes subscriptions\n- Always sets `reconnect_on_error: false` to prevent duplicate attempts","ref":"reconnection.html#layer-2-adapter-level-supervision-process-recovery"},{"type":"extras","title":"Gun Process Ownership Model - Reconnection Architecture Guide","doc":"Gun creates a dedicated process for each WebSocket connection with flexible ownership:\n\n```\n┌─────────────────┐\n│   Client        │\n│  GenServer      │─── owns ──→ ┌─────────────┐\n└─────────────────┘             │ Gun Process │\n                                └─────────────┘\n```\n\n**Key Features:**\n1. The process that creates the Gun connection owns it initially\n2. Gun sends all messages to the owning process\n3. Ownership can be transferred between processes if needed\n4. If owner dies, Gun process terminates\n5. New connections require new Gun processes\n\nThis ownership model enables our Client GenServer to directly receive and handle all Gun messages, including WebSocket frames and heartbeats.","ref":"reconnection.html#gun-process-ownership-model"},{"type":"extras","title":"When to Use Each Reconnection Mechanism - Reconnection Architecture Guide","doc":"","ref":"reconnection.html#when-to-use-each-reconnection-mechanism"},{"type":"extras","title":"Use Client-Level Reconnection (Standalone Pattern) - Reconnection Architecture Guide","doc":"For simple use cases without complex state requirements:\n\n```elixir\n# Client handles its own reconnection\n{:ok, client} = Client.connect(url)  # reconnect_on_error: true (default)\n```\n\n**Characteristics:**\n- Network failures trigger automatic reconnection\n- Connection state preserved\n- No authentication restoration\n- No subscription restoration\n- Suitable for read-only market data","ref":"reconnection.html#use-client-level-reconnection-standalone-pattern"},{"type":"extras","title":"Use Adapter-Level Supervision (Supervised Pattern) - Reconnection Architecture Guide","doc":"For production trading systems requiring full state restoration:\n\n```elixir\n# Adapter disables client reconnection\nconnect_opts = [\n  reconnect_on_error: false,  # Client stops cleanly on errors\n  heartbeat_config: %{...}    # Other options preserved\n]\n```\n\n**Characteristics:**\n- Adapter monitors Client process\n- Full state restoration on crashes\n- Authentication automatically restored\n- Subscriptions re-established\n- Cancel-on-disconnect protection maintained","ref":"reconnection.html#use-adapter-level-supervision-supervised-pattern"},{"type":"extras","title":"Message and Failure Flows - Reconnection Architecture Guide","doc":"","ref":"reconnection.html#message-and-failure-flows"},{"type":"extras","title":"Normal Operation Flow - Reconnection Architecture Guide","doc":"```\nUser Request\n    ↓\nAdapter GenServer\n    ↓\nClient GenServer\n    ↓\nGun Process ←──── WebSocket ────→ Exchange\n    ↓\nClient GenServer (receives Gun messages directly)\n    ↓\nAdapter GenServer\n    ↓\nUser Response\n```","ref":"reconnection.html#normal-operation-flow"},{"type":"extras","title":"Network Failure with Client Reconnection - Reconnection Architecture Guide","doc":"```\nNetwork Failure Detected\n    ↓\nGun Process notifies Client GenServer\n    ↓\nClient checks reconnect_on_error\n    ↓\nIf true: Exponential backoff retry\n    ↓\nCreate new Gun process (Client owns it)\n    ↓\nRe-establish WebSocket\n    ↓\nResume message flow\n```","ref":"reconnection.html#network-failure-with-client-reconnection"},{"type":"extras","title":"Process Crash with Adapter Supervision - Reconnection Architecture Guide","doc":"```\nClient Process Crash\n    ↓\nAdapter receives {:DOWN, ...}\n    ↓\nAdapter initiates reconnection\n    ↓\nCreate new Client (reconnect_on_error: false)\n    ↓\nNew Client creates Gun process (owns it)\n    ↓\nRestore authentication\n    ↓\nRe-establish subscriptions\n    ↓\nResume trading operations\n```","ref":"reconnection.html#process-crash-with-adapter-supervision"},{"type":"extras","title":"Architecture Decision Records - Reconnection Architecture Guide","doc":"","ref":"reconnection.html#architecture-decision-records"},{"type":"extras","title":"ADR-001: Dual-Layer Reconnection - Reconnection Architecture Guide","doc":"**Context:** Different use cases require different reconnection strategies.\n**Decision:** Implement reconnection at both Client and Adapter layers.\n**Consequences:** Clear separation of concerns, no duplicate attempts, flexible deployment.","ref":"reconnection.html#adr-001-dual-layer-reconnection"},{"type":"extras","title":"ADR-002: Configuration-Based Behavior - Reconnection Architecture Guide","doc":"**Context:** Need to prevent duplicate reconnection attempts in supervised scenarios.\n**Decision:** Use `reconnect_on_error` flag to control Client behavior.\n**Consequences:** Backward compatibility maintained, explicit configuration required.","ref":"reconnection.html#adr-002-configuration-based-behavior"},{"type":"extras","title":"ADR-003: Client GenServer Owns Gun Connection - Reconnection Architecture Guide","doc":"**Context:** Gun sends messages to the owning process; integrated heartbeat needs these messages.\n**Decision:** Client GenServer always owns the Gun connection it creates.\n**Consequences:** Direct message handling, simplified architecture, no message forwarding needed.","ref":"reconnection.html#adr-003-client-genserver-owns-gun-connection"},{"type":"extras","title":"ADR-004: Adapter Ownership of Reconnection Logic - Reconnection Architecture Guide","doc":"**Context:** Production systems need full state restoration beyond network recovery.\n**Decision:** Adapters always disable Client reconnection and handle it themselves.\n**Consequences:** Consistent pattern, predictable behavior, simplified debugging.","ref":"reconnection.html#adr-004-adapter-ownership-of-reconnection-logic"},{"type":"extras","title":"Performance Considerations - Reconnection Architecture Guide","doc":"1. **Connection Pooling**: Each reconnection creates a new Gun process. Consider connection pooling for high-frequency reconnection scenarios.\n\n2. **Exponential Backoff**: Both layers implement exponential backoff to prevent thundering herd problems.\n\n3. **State Restoration Cost**: Adapter-level reconnection has higher overhead due to authentication and subscription restoration.\n\n4. **Memory Usage**: Failed Gun processes are garbage collected. Monitor process count during extended outages.\n\n5. **Message Routing**: Client GenServer owns Gun connection, eliminating message forwarding overhead.","ref":"reconnection.html#performance-considerations"},{"type":"extras","title":"Best Practices - Reconnection Architecture Guide","doc":"1. **Always use supervised pattern for production trading**\n2. **Monitor both Client and Adapter processes**\n3. **Implement circuit breakers for persistent failures**\n4. **Log reconnection attempts with correlation IDs**\n5. **Test reconnection scenarios in staging environments**\n6. **Set appropriate timeout values for your latency requirements**\n7. **Ensure Client GenServer owns Gun connection for direct message handling**","ref":"reconnection.html#best-practices"},{"type":"extras","title":"Implementation Details - Reconnection Architecture Guide","doc":"","ref":"reconnection.html#implementation-details"},{"type":"extras","title":"Client Configuration - Reconnection Architecture Guide","doc":"```elixir\ndefmodule WebsockexAdapter.Config do\n  defstruct [\n    # ... other fields ...\n    reconnect_on_error: true,  # Default: enabled for standalone use\n    # ... other fields ...\n  ]\nend\n```","ref":"reconnection.html#client-configuration"},{"type":"extras","title":"Adapter Pattern - Reconnection Architecture Guide","doc":"```elixir\n# In adapter's do_connect function\nconnect_opts = [\n  heartbeat_config: %{\n    type: :deribit,\n    interval: heartbeat_interval\n  },\n  reconnect_on_error: false  # Critical: prevents duplicate reconnection\n]\n\n{:ok, client} = Client.connect(url, connect_opts)\n```","ref":"reconnection.html#adapter-pattern"},{"type":"extras","title":"Monitor Pattern - Reconnection Architecture Guide","doc":"```elixir\n# Adapter monitors Client for process-level failures\nref = Process.monitor(client.server_pid)\n\n# Handle Client death\ndef handle_info({:DOWN, ^ref, :process, _pid, reason}, state) do\n  # Initiate adapter-level reconnection with full state restoration\nend\n```","ref":"reconnection.html#monitor-pattern"},{"type":"extras","title":"Related Documentation - Reconnection Architecture Guide","doc":"- [Building Adapters Guide](../guides/building_adapters.md)\n- [Troubleshooting Reconnection](../guides/troubleshooting_reconnection.md)\n- [Gun Integration Guide](../gun_integration.md)\n- [Client Module Documentation](../../lib/websockex_adapter/client.ex)\n- [DeribitGenServerAdapter Example](../../lib/websockex_adapter/examples/deribit_genserver_adapter.ex)","ref":"reconnection.html#related-documentation"},{"type":"extras","title":"Gun Integration Guide","doc":"# Gun Integration Guide\n\nThis guide covers the integration of the [Gun](https://github.com/ninenines/gun) HTTP/WebSocket client in WebsockexAdapter, focusing on connection management, process monitoring, and ownership transfer.","ref":"gun_integration.html"},{"type":"extras","title":"Table of Contents - Gun Integration Guide","doc":"- [Overview](#overview)\n- [Process Monitoring vs. Linking](#process-monitoring-vs-linking)\n- [Using Gun's Await Functions](#using-guns-await-functions)\n- [Ownership Transfer](#ownership-transfer)\n- [Best Practices](#best-practices)","ref":"gun_integration.html#table-of-contents"},{"type":"extras","title":"Overview - Gun Integration Guide","doc":"WebsockexAdapter uses Gun as its underlying transport layer for WebSocket connections. Gun provides robust HTTP and WebSocket protocol implementation with features like:\n- HTTP/1.1, HTTP/2, and WebSocket support\n- Automatic reconnection capabilities\n- Comprehensive TLS options\n- Message streaming and multiplexing\n\nThe `WebsockexAdapter.Client` module will be refactored to a GenServer that owns the Gun connection and manages message routing.","ref":"gun_integration.html#overview"},{"type":"extras","title":"Process Monitoring vs. Linking - Gun Integration Guide","doc":"Gun gives developers the choice between using process links and monitors for tracking connection processes:","ref":"gun_integration.html#process-monitoring-vs-linking"},{"type":"extras","title":"Why WebsockexAdapter Uses Monitors - Gun Integration Guide","doc":"WebsockexAdapter uses Erlang's process monitoring (`Process.monitor/1`) instead of process linking for tracking Gun connections for several reasons:\n\n1. **Resilience**: If a Gun process crashes, the monitoring process receives a message rather than crashing itself\n2. **Control**: More granular control over error handling and recovery\n3. **Ownership Transfers**: Easier to manage process relationships during ownership changes","ref":"gun_integration.html#why-websockexadapter-uses-monitors"},{"type":"extras","title":"Current Implementation in Client - Gun Integration Guide","doc":"```elixir\n# In WebsockexAdapter.Client.connect/2\n{:ok, gun_pid} = :gun.open(host_charlist, port, gun_opts)\nmonitor_ref = Process.monitor(gun_pid)\n\n%Client{\n  gun_pid: gun_pid,\n  monitor_ref: monitor_ref,\n  state: :connecting,\n  # ...\n}\n```","ref":"gun_integration.html#current-implementation-in-client"},{"type":"extras","title":"Using Gun's Await Functions - Gun Integration Guide","doc":"Gun provides await functions for synchronous operations, but they require careful handling of the monitor reference.","ref":"gun_integration.html#using-gun-s-await-functions"},{"type":"extras","title":"The Monitor Reference Requirement - Gun Integration Guide","doc":"Gun's await functions check that the calling process has a monitor on the Gun connection:\n\n```elixir\n# Current usage in WebsockexAdapter.Client\ndefp await_websocket_upgrade(gun_pid, stream_ref, timeout, monitor_ref) do\n  case :gun.await(gun_pid, stream_ref, timeout, monitor_ref) do\n    {:upgrade, [<<\"websocket\">>], _headers} -> :ok\n    {:error, reason} -> {:error, reason}\n  end\nend\n```","ref":"gun_integration.html#the-monitor-reference-requirement"},{"type":"extras","title":"Common Pitfalls - Gun Integration Guide","doc":"1. **Missing Monitor Reference**: Calling await without the monitor reference will fail\n2. **Wrong Monitor Reference**: Using a monitor reference from a different connection\n3. **Monitor After Connect**: The monitor must be established before calling await functions","ref":"gun_integration.html#common-pitfalls"},{"type":"extras","title":"Ownership Transfer - Gun Integration Guide","doc":"One of Gun's most powerful features is the ability to transfer connection ownership between processes. This is crucial for WebsockexAdapter's upcoming architecture where the Client GenServer needs to own the connection for message routing.","ref":"gun_integration.html#ownership-transfer"},{"type":"extras","title":"When to Transfer Ownership - Gun Integration Guide","doc":"Ownership transfer is useful when:\n- Client GenServer needs to receive Gun messages for integrated heartbeat processing\n- Reconnection creates a new Gun process that needs proper ownership\n- Moving connections between supervision trees","ref":"gun_integration.html#when-to-transfer-ownership"},{"type":"extras","title":"Implementation for Integrated Heartbeat - Gun Integration Guide","doc":"```elixir\ndefmodule WebsockexAdapter.Client do\n  use GenServer\n  \n  # Client GenServer owns the Gun connection\n  def init(config) do\n    {:ok, gun_pid} = :gun.open(host, port, opts)\n    monitor_ref = Process.monitor(gun_pid)\n    \n    # Client GenServer (self()) owns the connection\n    # All Gun messages come to this process\n    \n    state = %{\n      gun_pid: gun_pid,\n      monitor_ref: monitor_ref,\n      heartbeat_manager: nil,\n      # ...\n    }\n    \n    {:ok, state}\n  end\n  \n  # Route Gun messages to appropriate handlers\n  def handle_info({:gun_ws, gun_pid, stream_ref, frame}, state) do\n    case MessageHandler.parse(frame) do\n      {:heartbeat, msg} ->\n        # Process heartbeat directly in Client\n        handle_heartbeat_message(state, msg)\n      {:user_message, msg} ->\n        # Forward to user handler\n        send(state.user_handler, {:websocket_message, msg})\n    end\n    {:noreply, state}\n  end\nend\n```","ref":"gun_integration.html#implementation-for-integrated-heartbeat"},{"type":"extras","title":"Reconnection Flow with Ownership - Gun Integration Guide","doc":"```elixir\ndef handle_info({:DOWN, ref, :process, pid, reason}, state) do\n  if state.monitor_ref == ref and state.gun_pid == pid do\n    # Connection lost, trigger reconnection\n    case Reconnection.reconnect(state.config) do\n      {:ok, new_gun_pid} ->\n        # Client GenServer already owns the new connection\n        # because Reconnection was called from this process\n        new_monitor_ref = Process.monitor(new_gun_pid)\n        \n        new_state = %{state | \n          gun_pid: new_gun_pid,\n          monitor_ref: new_monitor_ref\n        }\n        \n        # Resume integrated heartbeat processing\n        resume_heartbeat_processing(new_state)\n        \n        {:noreply, new_state}\n    end\n  end\nend\n```","ref":"gun_integration.html#reconnection-flow-with-ownership"},{"type":"extras","title":"Best Practices - Gun Integration Guide","doc":"","ref":"gun_integration.html#best-practices"},{"type":"extras","title":"1. Always Use Monitors - Gun Integration Guide","doc":"```elixir\n# Good - WebsockexAdapter.Client pattern\n{:ok, gun_pid} = :gun.open(host, port, opts)\nmonitor_ref = Process.monitor(gun_pid)\n\n# Bad - no visibility into connection failures\n{:ok, gun_pid} = :gun.open(host, port, opts)\n```","ref":"gun_integration.html#1-always-use-monitors"},{"type":"extras","title":"2. Handle Monitor Messages - Gun Integration Guide","doc":"```elixir\n# In Client GenServer\ndef handle_info({:DOWN, ref, :process, pid, reason}, state) do\n  cond do\n    ref == state.monitor_ref ->\n      handle_connection_down(reason, state)\n    true ->\n      {:noreply, state}\n  end\nend\n```","ref":"gun_integration.html#2-handle-monitor-messages"},{"type":"extras","title":"3. Client GenServer Owns Gun Connection - Gun Integration Guide","doc":"The Client GenServer must own the Gun connection to receive messages:\n\n```elixir\ndefmodule WebsockexAdapter.Client do\n  use GenServer\n  \n  # Gun messages come to the GenServer process\n  def handle_info({:gun_ws, _, _, _} = msg, state) do\n    route_message(msg, state)\n  end\n  \n  def handle_info({:gun_response, _, _, _} = msg, state) do\n    route_message(msg, state)\n  end\nend\n```","ref":"gun_integration.html#3-client-genserver-owns-gun-connection"},{"type":"extras","title":"4. Clean Reconnection - Gun Integration Guide","doc":"```elixir\ndefp handle_reconnection(state) do\n  # Old connection cleanup\n  Process.demonitor(state.monitor_ref, [:flush])\n  \n  # Create new connection (Client GenServer owns it)\n  case establish_new_connection(state.config) do\n    {:ok, gun_pid} ->\n      monitor_ref = Process.monitor(gun_pid)\n      %{state | gun_pid: gun_pid, monitor_ref: monitor_ref}\n  end\nend\n```","ref":"gun_integration.html#4-clean-reconnection"},{"type":"extras","title":"5. Test Connection Failures - Gun Integration Guide","doc":"Always test how your application handles:\n- Gun process crashes during active trading\n- Network disconnections during heartbeat sequences\n- Reconnection with subscription restoration\n- Message routing after reconnection","ref":"gun_integration.html#5-test-connection-failures"},{"type":"extras","title":"Summary - Gun Integration Guide","doc":"Gun's process monitoring and ownership features are critical for WebsockexAdapter's architecture. By having the Client GenServer own the Gun connection, we enable:\n- Integrated heartbeat processing and user message handling\n- Seamless reconnection with state preservation\n- Reliable heartbeat handling for financial trading\n- Clean separation of concerns between modules\n\nThe key insight is that Gun sends messages to the process that owns the connection, making the Client GenServer refactor essential for production-grade WebSocket handling.","ref":"gun_integration.html#summary"},{"type":"extras","title":"WebsockexAdapter Supervision Strategy","doc":"# WebsockexAdapter Supervision Strategy","ref":"supervision_strategy.html"},{"type":"extras","title":"Overview - WebsockexAdapter Supervision Strategy","doc":"WebsockexAdapter provides optional supervision for WebSocket client connections, ensuring resilience and automatic recovery from failures. This is critical for financial trading systems where connection stability directly impacts order execution and risk management.\n\n**Important**: As a library, WebsockexAdapter does not start any supervisors automatically. You must explicitly add supervision to your application's supervision tree when needed.","ref":"supervision_strategy.html#overview"},{"type":"extras","title":"Architecture - WebsockexAdapter Supervision Strategy","doc":"```\nYour Application Supervisor\n    ├── WebsockexAdapter.ClientSupervisor (Optional DynamicSupervisor)\n    │       ├── Client GenServer 1\n    │       ├── Client GenServer 2\n    │       └── Client GenServer N\n    └── Your other children...\n```","ref":"supervision_strategy.html#architecture"},{"type":"extras","title":"Key Components - WebsockexAdapter Supervision Strategy","doc":"","ref":"supervision_strategy.html#key-components"},{"type":"extras","title":"1. ClientSupervisor (WebsockexAdapter.ClientSupervisor) - WebsockexAdapter Supervision Strategy","doc":"- DynamicSupervisor for managing client connections\n- Restart strategy: `:one_for_one` (isolated failures)\n- Maximum 10 restarts in 60 seconds (configurable)\n- Each client runs independently","ref":"supervision_strategy.html#1-clientsupervisor-websockexadapter-clientsupervisor"},{"type":"extras","title":"2. Client GenServer (WebsockexAdapter.Client) - WebsockexAdapter Supervision Strategy","doc":"- Manages individual WebSocket connections\n- Handles Gun process ownership and message routing\n- Integrated heartbeat handling\n- Automatic reconnection on network failures","ref":"supervision_strategy.html#2-client-genserver-websockexadapter-client"},{"type":"extras","title":"Usage Patterns - WebsockexAdapter Supervision Strategy","doc":"","ref":"supervision_strategy.html#usage-patterns"},{"type":"extras","title":"Pattern 1: No Supervision (Simple/Testing) - WebsockexAdapter Supervision Strategy","doc":"```elixir\n# Direct connection without supervision\n{:ok, client} = WebsockexAdapter.Client.connect(\"wss://example.com\")\n\n# Use the client\nWebsockexAdapter.Client.send_message(client, \"Hello\")\n\n# Clean up when done\nWebsockexAdapter.Client.close(client)\n```","ref":"supervision_strategy.html#pattern-1-no-supervision-simple-testing"},{"type":"extras","title":"Pattern 2: Using ClientSupervisor - WebsockexAdapter Supervision Strategy","doc":"First, add the supervisor to your application:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n  \n  def start(_type, _args) do\n    children = [\n      # Add the WebsockexAdapter supervisor\n      WebsockexAdapter.ClientSupervisor,\n      # Your other children...\n    ]\n    \n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```\n\nThen create supervised connections:\n\n```elixir\n# Basic supervised connection\n{:ok, client} = WebsockexAdapter.ClientSupervisor.start_client(\"wss://example.com\")\n\n# With configuration\n{:ok, client} = WebsockexAdapter.ClientSupervisor.start_client(\"wss://example.com\",\n  retry_count: 10,\n  heartbeat_config: %{type: :deribit, interval: 30_000}\n)\n```","ref":"supervision_strategy.html#pattern-2-using-clientsupervisor"},{"type":"extras","title":"Pattern 3: Direct Client Supervision - WebsockexAdapter Supervision Strategy","doc":"Add individual clients directly to your supervision tree:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n  \n  def start(_type, _args) do\n    children = [\n      # Supervise individual clients\n      {WebsockexAdapter.Client, [\n        url: \"wss://exchange1.com\",\n        id: :exchange1_client,\n        heartbeat_config: %{type: :deribit, interval: 30_000}\n      ]},\n      {WebsockexAdapter.Client, [\n        url: \"wss://exchange2.com\", \n        id: :exchange2_client\n      ]},\n      # Your other children...\n    ]\n    \n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```","ref":"supervision_strategy.html#pattern-3-direct-client-supervision"},{"type":"extras","title":"Restart Behavior - WebsockexAdapter Supervision Strategy","doc":"","ref":"supervision_strategy.html#restart-behavior"},{"type":"extras","title":"Transient Restart Strategy - WebsockexAdapter Supervision Strategy","doc":"- Clients are restarted only if they exit abnormally\n- Normal shutdowns (via `Client.close/1`) don't trigger restart\n- Crashes and connection failures trigger automatic restart","ref":"supervision_strategy.html#transient-restart-strategy"},{"type":"extras","title":"Failure Scenarios - WebsockexAdapter Supervision Strategy","doc":"1. **Network Disconnection**\n   - Client detects connection loss\n   - Attempts internal reconnection (configurable retries)\n   - If max retries exceeded, GenServer exits\n   - Supervisor restarts the client\n\n2. **Process Crash**\n   - Supervisor immediately detects exit\n   - Starts new client process\n   - Connection re-established from scratch\n\n3. **Heartbeat Failure**\n   - Client tracks heartbeat failures\n   - Closes connection after threshold\n   - Supervisor restarts for fresh connection","ref":"supervision_strategy.html#failure-scenarios"},{"type":"extras","title":"Production Considerations - WebsockexAdapter Supervision Strategy","doc":"","ref":"supervision_strategy.html#production-considerations"},{"type":"extras","title":"1. Resource Management - WebsockexAdapter Supervision Strategy","doc":"- Each supervised client consumes:\n  - 1 Erlang process (Client GenServer)\n  - 1 Gun connection process\n  - Associated memory for state and buffers","ref":"supervision_strategy.html#1-resource-management"},{"type":"extras","title":"2. Restart Limits - WebsockexAdapter Supervision Strategy","doc":"- Default: 10 restarts in 60 seconds\n- Prevents restart storms\n- Adjust based on expected failure patterns","ref":"supervision_strategy.html#2-restart-limits"},{"type":"extras","title":"3. Monitoring - WebsockexAdapter Supervision Strategy","doc":"```elixir\n# List all supervised clients\nclients = WebsockexAdapter.ClientSupervisor.list_clients()\n\n# Check client health\nhealth = WebsockexAdapter.Client.get_heartbeat_health(client)\n```","ref":"supervision_strategy.html#3-monitoring"},{"type":"extras","title":"4. Graceful Shutdown - WebsockexAdapter Supervision Strategy","doc":"```elixir\n# Stop a specific client\nWebsockexAdapter.ClientSupervisor.stop_client(pid)\n\n# Client won't be restarted (normal termination)\n```","ref":"supervision_strategy.html#4-graceful-shutdown"},{"type":"extras","title":"Best Practices - WebsockexAdapter Supervision Strategy","doc":"1. **Use Supervision for Production**\n   - Always use `ClientSupervisor.start_client/2` for production\n   - Direct connections only for testing/development\n\n2. **Configure Appropriate Timeouts**\n   - Set heartbeat intervals based on exchange requirements\n   - Configure retry counts for network conditions\n\n3. **Monitor Client Health**\n   - Implement health checks using `get_heartbeat_health/1`\n   - Set up alerts for excessive restarts\n\n4. **Handle Restart Events**\n   - Subscriptions may need re-establishment\n   - Authentication may need renewal\n   - Order state should be reconciled","ref":"supervision_strategy.html#best-practices"},{"type":"extras","title":"Example: Production Deribit Connection - WebsockexAdapter Supervision Strategy","doc":"```elixir\ndefmodule TradingSystem.DeribitConnection do\n  use GenServer\n  \n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n  \n  def init(opts) do\n    # Start supervised connection\n    url = \"wss://test.deribit.com/ws/api/v2\"\n    config = [\n      heartbeat_config: %{type: :deribit, interval: 30_000},\n      retry_count: 10,\n      retry_delay: 1000\n    ]\n    \n    {:ok, client} = WebsockexAdapter.ClientSupervisor.start_client(url, config)\n    \n    # Create adapter with supervised client\n    adapter = %WebsockexAdapter.Examples.DeribitAdapter{\n      client: client,\n      authenticated: false,\n      subscriptions: MapSet.new(),\n      client_id: opts[:client_id],\n      client_secret: opts[:client_secret]\n    }\n    \n    # Authenticate and subscribe\n    {:ok, adapter} = WebsockexAdapter.Examples.DeribitAdapter.authenticate(adapter)\n    {:ok, adapter} = WebsockexAdapter.Examples.DeribitAdapter.subscribe(adapter, [\n      \"book.BTC-PERPETUAL.raw\",\n      \"trades.BTC-PERPETUAL.raw\",\n      \"user.orders.BTC-PERPETUAL.raw\"\n    ])\n    \n    {:ok, %{adapter: adapter}}\n  end\n  \n  # Handle reconnection events\n  def handle_info({:gun_down, _, _, _, _}, state) do\n    # Log disconnection\n    Logger.warn(\"Deribit connection lost, supervisor will restart\")\n    {:noreply, state}\n  end\nend\n```","ref":"supervision_strategy.html#example-production-deribit-connection"},{"type":"extras","title":"Supervision Tree Visualization - WebsockexAdapter Supervision Strategy","doc":"```\nYourApp.Supervisor\n    ├── WebsockexAdapter.Application\n    │   └── WebsockexAdapter.ClientSupervisor\n    │       ├── Client_1 (Deribit Production)\n    │       ├── Client_2 (Deribit Test)\n    │       └── Client_3 (Binance)\n    └── YourApp.TradingEngine\n```\n\nThe supervision strategy ensures that WebSocket connections remain stable and automatically recover from failures, critical for 24/7 financial trading operations.","ref":"supervision_strategy.html#supervision-tree-visualization"},{"type":"extras","title":"WebsockexAdapter Stability Testing Guide","doc":"# WebsockexAdapter Stability Testing Guide\n\nThis guide covers the stability testing capabilities for WebsockexAdapter, specifically designed for production-grade WebSocket connections with financial APIs.","ref":"stability_testing.html"},{"type":"extras","title":"Overview - WebsockexAdapter Stability Testing Guide","doc":"The stability tests verify:\n- **Continuous heartbeat functionality** over extended periods\n- **Automatic reconnection** on failures\n- **Memory stability** under load\n- **Message handling performance**\n- **Network disruption recovery**","ref":"stability_testing.html#overview"},{"type":"extras","title":"Available Tests - WebsockexAdapter Stability Testing Guide","doc":"","ref":"stability_testing.html#available-tests"},{"type":"extras","title":"1. Development Stability Test (1 hour) - WebsockexAdapter Stability Testing Guide","doc":"Quick validation test for development environments.\n\n```bash\n# Run 1-hour test\nmix stability_test\n\n# Or directly\nmix test --only stability_dev test/websockex_adapter/examples/deribit_stability_dev_test.exs\n```","ref":"stability_testing.html#1-development-stability-test-1-hour"},{"type":"extras","title":"2. Production Stability Test (24 hours) - WebsockexAdapter Stability Testing Guide","doc":"Comprehensive test for production validation.\n\n```bash\n# Run 24-hour test\nmix stability_test --full\n\n# Or directly\nmix test --only stability test/websockex_adapter/examples/deribit_stability_test.exs\n```","ref":"stability_testing.html#2-production-stability-test-24-hours"},{"type":"extras","title":"Prerequisites - WebsockexAdapter Stability Testing Guide","doc":"Set your Deribit credentials:\n```bash\nexport DERIBIT_CLIENT_ID=\"your_client_id\"\nexport DERIBIT_CLIENT_SECRET=\"your_client_secret\"\n```","ref":"stability_testing.html#prerequisites"},{"type":"extras","title":"Test Components - WebsockexAdapter Stability Testing Guide","doc":"","ref":"stability_testing.html#test-components"},{"type":"extras","title":"StabilityMonitor - WebsockexAdapter Stability Testing Guide","doc":"Tracks key metrics during the test:\n- Heartbeat count and intervals\n- Reconnection events\n- Error occurrences\n- Message throughput","ref":"stability_testing.html#stabilitymonitor"},{"type":"extras","title":"MessageHandler - WebsockexAdapter Stability Testing Guide","doc":"Custom WebSocket handler that:\n- Records heartbeat messages\n- Tracks market data updates\n- Logs connection/disconnection events","ref":"stability_testing.html#messagehandler"},{"type":"extras","title":"Test Output - WebsockexAdapter Stability Testing Guide","doc":"The tests provide:\n1. **Periodic Status Reports** (every minute/5 minutes)\n2. **Final Assessment Report** with:\n   - Heartbeat success rate\n   - Connection stability metrics\n   - Error analysis\n   - Overall pass/fail assessment\n\nExample output:\n```\n📊 === STABILITY TEST STATUS REPORT ===\n⏱️  Runtime: 2.5 hours\n💓 Heartbeats: 300 (120.0/hour)\n🔄 Reconnections: 0\n❌ Errors: 0\n📨 Messages: 15423\n🕐 Last heartbeat: 15s ago\n=====================================\n```","ref":"stability_testing.html#test-output"},{"type":"extras","title":"Success Criteria - WebsockexAdapter Stability Testing Guide","doc":"","ref":"stability_testing.html#success-criteria"},{"type":"extras","title":"Development Test (1 hour) - WebsockexAdapter Stability Testing Guide","doc":"- Heartbeat success rate > 90%\n- Reconnections < 2\n- Errors < 5","ref":"stability_testing.html#development-test-1-hour"},{"type":"extras","title":"Production Test (24 hours) - WebsockexAdapter Stability Testing Guide","doc":"- Heartbeat success rate > 95%\n- Reconnection rate < 1 per hour\n- Error rate < 2 per hour","ref":"stability_testing.html#production-test-24-hours"},{"type":"extras","title":"Monitoring During Tests - WebsockexAdapter Stability Testing Guide","doc":"The tests automatically:\n- Verify adapter process health every 30 seconds\n- Re-authenticate if connection is lost\n- Restore subscriptions after reconnection\n- Generate detailed logs for debugging","ref":"stability_testing.html#monitoring-during-tests"},{"type":"extras","title":"Report Files - WebsockexAdapter Stability Testing Guide","doc":"Tests generate timestamped reports:\n```\nstability_report_2025-01-23T12:00:00Z.txt\n```","ref":"stability_testing.html#report-files"},{"type":"extras","title":"Troubleshooting - WebsockexAdapter Stability Testing Guide","doc":"Common issues:\n1. **Missing credentials**: Set DERIBIT_CLIENT_ID and DERIBIT_CLIENT_SECRET\n2. **Network issues**: Ensure stable internet connection\n3. **Rate limits**: Test uses minimal API calls to avoid limits","ref":"stability_testing.html#troubleshooting"},{"type":"extras","title":"Integration with CI/CD - WebsockexAdapter Stability Testing Guide","doc":"For CI environments, use the 1-hour test:\n```yaml\ntest:\n  script:\n    - mix deps.get\n    - mix compile\n    - mix stability_test\n```\n\nFor pre-production validation, run the full 24-hour test in a staging environment.","ref":"stability_testing.html#integration-with-ci-cd"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}